@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -108,7 +108,7 @@ gst_curl_ftp_sink_class_init (GstCurlFtpSinkClass * klass)
       "curl ftp sink element");
   GST_DEBUG_OBJECT (klass, "class_init");
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "Curl ftp sink",
       "Sink/Network",
       "Upload data over FTP protocol using libcurl",
