@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -126,7 +126,6 @@ static gboolean gst_faac_configure_source_pad (GstFaac * faac,
     GstAudioInfo * info);
 static GstCaps *gst_faac_getcaps (GstAudioEncoder * enc, GstCaps * filter);
 
-static gboolean gst_faac_start (GstAudioEncoder * enc);
 static gboolean gst_faac_stop (GstAudioEncoder * enc);
 static gboolean gst_faac_set_format (GstAudioEncoder * enc,
     GstAudioInfo * info);
@@ -207,7 +206,6 @@ gst_faac_class_init (GstFaacClass * klass)
       "Free MPEG-2/4 AAC encoder",
       "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
-  base_class->start = GST_DEBUG_FUNCPTR (gst_faac_start);
   base_class->stop = GST_DEBUG_FUNCPTR (gst_faac_stop);
   base_class->set_format = GST_DEBUG_FUNCPTR (gst_faac_set_format);
   base_class->handle_frame = GST_DEBUG_FUNCPTR (gst_faac_handle_frame);
@@ -259,15 +257,6 @@ gst_faac_close_encoder (GstFaac * faac)
   faac->handle = NULL;
 }
 
-static gboolean
-gst_faac_start (GstAudioEncoder * enc)
-{
-  GstFaac *faac = GST_FAAC (enc);
-
-  GST_DEBUG_OBJECT (faac, "start");
-  return TRUE;
-}
-
 static gboolean
 gst_faac_stop (GstAudioEncoder * enc)
 {
