@@ -110,20 +110,20 @@ static gboolean
 int_from_string (gchar * ptr, gchar ** endptr, gint * val)
 {
   gchar *end;
-  glong ret;
+  gint64 ret;
 
   g_return_val_if_fail (ptr != NULL, FALSE);
   g_return_val_if_fail (val != NULL, FALSE);
 
   errno = 0;
-  ret = strtol (ptr, &end, 10);
-  if ((errno == ERANGE && (ret == LONG_MAX || ret == LONG_MIN))
+  ret = g_ascii_strtoll (ptr, &end, 10);
+  if ((errno == ERANGE && (ret == G_MAXINT64 || ret == G_MININT64))
       || (errno != 0 && ret == 0)) {
     GST_WARNING ("%s", g_strerror (errno));
     return FALSE;
   }
 
-  if (ret > G_MAXINT) {
+  if (ret > G_MAXINT || ret < G_MININT) {
     GST_WARNING ("%s", g_strerror (ERANGE));
     return FALSE;
   }
@@ -146,7 +146,7 @@ double_from_string (gchar * ptr, gchar ** endptr, gdouble * val)
   g_return_val_if_fail (val != NULL, FALSE);
 
   errno = 0;
-  ret = strtod (ptr, &end);
+  ret = g_ascii_strtod (ptr, &end);
   if ((errno == ERANGE && (ret == HUGE_VAL || ret == -HUGE_VAL))
       || (errno != 0 && ret == 0)) {
     GST_WARNING ("%s", g_strerror (errno));
@@ -161,7 +161,7 @@ double_from_string (gchar * ptr, gchar ** endptr, gdouble * val)
   if (endptr)
     *endptr = end;
 
-  *val = (gint) ret;
+  *val = (gdouble) ret;
 
   return end != ptr;
 }
@@ -169,7 +169,7 @@ double_from_string (gchar * ptr, gchar ** endptr, gdouble * val)
 static gboolean
 parse_attributes (gchar ** ptr, gchar ** a, gchar ** v)
 {
-  gchar *end, *p;
+  gchar *end=NULL, *p;
 
   g_return_val_if_fail (ptr != NULL, FALSE);
   g_return_val_if_fail (*ptr != NULL, FALSE);
@@ -180,6 +180,19 @@ parse_attributes (gchar ** ptr, gchar ** a, gchar ** v)
 
   *a = *ptr;
   end = p = g_utf8_strchr (*ptr, -1, ',');
+  if(end){
+	gchar *q = g_utf8_strchr (*ptr, -1, '"');
+	if(q && q<end){
+	  /* special case, such as CODECS="avc1.77.30, mp4a.40.2" */
+	  q = g_utf8_next_char (q);
+	  if(q){
+		q = g_utf8_strchr (q, -1, '"');
+	  }
+	  if(q){
+		end = p = g_utf8_strchr (q, -1, ',');
+	  }
+	}
+  }
   if (end) {
     do {
       end = g_utf8_next_char (end);
@@ -215,6 +228,53 @@ gst_m3u8_compare_playlist_by_bitrate (gconstpointer a, gconstpointer b)
   return ((GstM3U8 *) (a))->bandwidth - ((GstM3U8 *) (b))->bandwidth;
 }
 
+static gint
+hex_char_to_int (const gchar * v)
+{
+  switch (*v) {
+    case '0':
+      return 0;
+    case '1':
+      return 1;
+    case '2':
+      return 2;
+    case '3':
+      return 3;
+    case '4':
+      return 4;
+    case '5':
+      return 5;
+    case '6':
+      return 6;
+    case '7':
+      return 7;
+    case '8':
+      return 8;
+    case '9':
+      return 9;
+    case 'A':
+    case 'a':
+      return 0xa;
+    case 'B':
+    case 'b':
+      return 0xb;
+    case 'C':
+    case 'c':
+      return 0xc;
+    case 'D':
+    case 'd':
+      return 0xd;
+    case 'E':
+    case 'e':
+      return 0xe;
+    case 'F':
+    case 'f':
+      return 0xf;
+    default:
+      return -1;
+  }
+}
+
 /*
  * @data: a m3u8 playlist text data, taking ownership
  */
@@ -226,6 +286,8 @@ gst_m3u8_update (GstM3U8 * self, gchar * data, gboolean * updated)
   gchar *title, *end;
 //  gboolean discontinuity;
   GstM3U8 *list;
+  gboolean have_iv = FALSE;
+  guint8 iv[16] = { 0, };
 
   g_return_val_if_fail (self != NULL, FALSE);
   g_return_val_if_fail (data != NULL, FALSE);
@@ -302,8 +364,12 @@ gst_m3u8_update (GstM3U8 * self, gchar * data, gboolean * updated)
         /* set encryption params */
         file->key = g_strdup (self->key);
         if (file->key) {
-          guint8 *iv = file->iv + 12;
-          GST_WRITE_UINT32_BE (iv, file->sequence);
+          if (have_iv) {
+            memcpy (file->iv, iv, sizeof (iv));
+          } else {
+            guint8 *iv = file->iv + 12;
+            GST_WRITE_UINT32_BE (iv + 12, file->sequence);
+          }
         }
 
         duration = 0;
@@ -339,7 +405,7 @@ gst_m3u8_update (GstM3U8 * self, gchar * data, gboolean * updated)
         } else if (g_str_equal (a, "RESOLUTION")) {
           if (!int_from_string (v, &v, &list->width))
             GST_WARNING ("Error while reading RESOLUTION width");
-          if (!v || *v != '=') {
+          if (!v || *v != 'x') {
             GST_WARNING ("Missing height");
           } else {
             v = g_utf8_next_char (v);
@@ -366,6 +432,11 @@ gst_m3u8_update (GstM3U8 * self, gchar * data, gboolean * updated)
       gchar *v, *a;
 
       data = data + 11;
+
+      /* IV and KEY are only valid until the next #EXT-X-KEY */
+      have_iv = FALSE;
+      g_free (self->key);
+      self->key = NULL;
       while (data && parse_attributes (&data, &a, &v)) {
         if (g_str_equal (a, "URI")) {
           gchar *key = g_strdup (v);
@@ -380,6 +451,38 @@ gst_m3u8_update (GstM3U8 * self, gchar * data, gboolean * updated)
 
           self->key = uri_join (self->uri, key);
           g_free (keyp);
+        } else if (g_str_equal (a, "IV")) {
+          gchar *ivp = v;
+          gint i;
+
+          if (strlen (ivp) < 32 + 2 || (!g_str_has_prefix (ivp, "0x")
+                  && !g_str_has_prefix (ivp, "0X"))) {
+            GST_WARNING ("Can't read IV");
+            continue;
+          }
+
+          ivp += 2;
+          for (i = 0; i < 16; i++) {
+            gint h, l;
+
+            h = hex_char_to_int (ivp++);
+            l = hex_char_to_int (ivp++);
+            if (h == -1 || l == -1) {
+              i = -1;
+              break;
+            }
+            iv[i] = (h << 4) | l;
+          }
+          if (i == -1) {
+            GST_WARNING ("Can't read IV");
+            continue;
+          }
+          have_iv = TRUE;
+        } else if (g_str_equal (a, "METHOD")) {
+          if (!g_str_equal (v, "AES-128")) {
+            GST_WARNING ("Encryption method %s not supported", v);
+            continue;
+          }
         }
       }
     } else if (g_str_has_prefix (data, "#EXTINF:")) {
@@ -705,7 +808,15 @@ uri_join (const gchar * uri1, const gchar * uri2)
   uri_copy = g_strdup (uri1);
   if (uri2[0] != '/') {
     /* uri2 is a relative uri2 */
-    tmp = g_utf8_strrchr (uri_copy, -1, '/');
+    /* look for query params */
+    tmp = g_utf8_strchr (uri_copy, -1, '?');
+    if (tmp) {
+      /* find last / char, ignoring query params */
+      tmp = g_utf8_strrchr (uri_copy, tmp - uri_copy, '/');
+    } else {
+      /* find last / char in URL */
+      tmp = g_utf8_strrchr (uri_copy, -1, '/');
+    }
     if (!tmp) {
       GST_WARNING ("Can't build a valid uri_copy");
       goto out;
