@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-mimdec
@@ -85,7 +85,7 @@ gst_mim_dec_class_init (GstMimDecClass * klass)
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (gstelement_class,
       "Mimic Decoder",
       "Codec/Decoder/Video",
       "MSN Messenger compatible Mimic video decoder element",
@@ -353,7 +353,6 @@ gst_mim_dec_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_STREAM_START:
     case GST_EVENT_FLUSH_STOP:
     case GST_EVENT_EOS:
-      mimdec->need_segment = FALSE;
       gst_adapter_clear (mimdec->adapter);
       break;
     default:
