@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -142,7 +142,7 @@ gst_mplex_class_init (GstMplexClass * klass)
       GST_DEBUG_FUNCPTR (gst_mplex_request_new_pad);
   element_class->release_pad = GST_DEBUG_FUNCPTR (gst_mplex_release_pad);
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "mplex video multiplexer", "Codec/Muxer",
       "High-quality MPEG/DVD/SVCD/VCD video/audio multiplexer",
       "Andrew Stevens <andrew.stevens@nexgo.de>\n"
