@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
@@ -88,7 +88,7 @@ gst_musepackdec_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_metadata (element_class, "Musepack decoder",
+  gst_element_class_set_static_metadata (element_class, "Musepack decoder",
       "Codec/Decoder/Audio",
       "Musepack decoder", "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 }
