@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -139,7 +139,7 @@ gst_opus_parse_handle_frame (GstBaseParse * base,
 {
   GstOpusParse *parse;
   guint8 *data;
-  gint size;
+  gsize size;
   guint32 packet_size;
   int ret = FALSE;
   const unsigned char *frames[48];
@@ -214,8 +214,8 @@ gst_opus_parse_handle_frame (GstBaseParse * base,
     size = payload_offset;
   }
 
-  GST_DEBUG_OBJECT (parse, "Got Opus packet at offset %d, %d bytes", *skip,
-      size);
+  GST_DEBUG_OBJECT (parse,
+      "Got Opus packet at offset %d, %" G_GSIZE_FORMAT " bytes", *skip, size);
   ret = TRUE;
 
 beach:
