@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -86,6 +86,7 @@ static void gst_rtmp_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_rtmp_src_finalize (GObject * object);
 
+static gboolean gst_rtmp_src_unlock (GstBaseSrc * src);
 static gboolean gst_rtmp_src_stop (GstBaseSrc * src);
 static gboolean gst_rtmp_src_start (GstBaseSrc * src);
 static gboolean gst_rtmp_src_is_seekable (GstBaseSrc * src);
@@ -136,6 +137,7 @@ gst_rtmp_src_class_init (GstRTMPSrcClass * klass)
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_rtmp_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_rtmp_src_stop);
+  gstbasesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_rtmp_src_unlock);
   gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR (gst_rtmp_src_is_seekable);
   gstbasesrc_class->prepare_seek_segment =
       GST_DEBUG_FUNCPTR (gst_rtmp_src_prepare_seek_segment);
@@ -503,11 +505,6 @@ gst_rtmp_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
     return FALSE;
   }
 
-  if (!src->seekable) {
-    GST_LOG_OBJECT (src, "Not a seekable stream");
-    return FALSE;
-  }
-
   if (!src->rtmp) {
     GST_LOG_OBJECT (src, "Not connected yet");
     return FALSE;
@@ -519,6 +516,11 @@ gst_rtmp_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   if (src->cur_offset == 0 && segment->start == 0)
     return TRUE;
 
+  if (!src->seekable) {
+    GST_LOG_OBJECT (src, "Not a seekable stream");
+    return FALSE;
+  }
+
   src->last_timestamp = GST_CLOCK_TIME_NONE;
   if (!RTMP_SendSeek (src->rtmp, segment->start / GST_MSECOND)) {
     GST_ERROR_OBJECT (src, "Seeking failed");
@@ -553,7 +555,6 @@ gst_rtmp_src_start (GstBaseSrc * basesrc)
 
   src->cur_offset = 0;
   src->last_timestamp = 0;
-  src->seekable = TRUE;
   src->discont = TRUE;
 
   uri_copy = g_strdup (src->uri);
@@ -567,6 +568,8 @@ gst_rtmp_src_start (GstBaseSrc * basesrc)
     src->rtmp = NULL;
     return FALSE;
   }
+  src->seekable = !(src->rtmp->Link.lFlags & RTMP_LF_LIVE);
+  GST_INFO_OBJECT (src, "seekable %d", src->seekable);
 
   /* open if required */
   if (!RTMP_IsConnected (src->rtmp)) {
@@ -584,6 +587,23 @@ gst_rtmp_src_start (GstBaseSrc * basesrc)
 
 #undef STR2AVAL
 
+static gboolean
+gst_rtmp_src_unlock (GstBaseSrc * basesrc)
+{
+  GstRTMPSrc *rtmpsrc = GST_RTMP_SRC (basesrc);
+
+  GST_DEBUG_OBJECT (rtmpsrc, "unlock");
+
+  /* This closes the socket, which means that any pending socket calls
+   * error out. */
+  if (rtmpsrc->rtmp) {
+    RTMP_Close (rtmpsrc->rtmp);
+  }
+
+  return TRUE;
+}
+
+
 static gboolean
 gst_rtmp_src_stop (GstBaseSrc * basesrc)
 {
@@ -592,7 +612,6 @@ gst_rtmp_src_stop (GstBaseSrc * basesrc)
   src = GST_RTMP_SRC (basesrc);
 
   if (src->rtmp) {
-    RTMP_Close (src->rtmp);
     RTMP_Free (src->rtmp);
     src->rtmp = NULL;
   }
