@@ -255,7 +255,6 @@ gst_mss_demux_stream_new (GstMssDemux * mssdemux,
 static void
 gst_mss_demux_stream_free (GstMssDemuxStream * stream)
 {
-  gst_download_rate_deinit (&stream->download_rate);
   if (stream->download_task) {
     if (GST_TASK_STATE (stream->download_task) != GST_TASK_STOPPED) {
       GST_DEBUG_OBJECT (stream->parent, "Leaving streaming task %s:%s",
@@ -273,12 +272,12 @@ gst_mss_demux_stream_free (GstMssDemuxStream * stream)
     stream->download_task = NULL;
   }
 
+  gst_download_rate_deinit (&stream->download_rate);
   if (stream->pending_newsegment) {
     gst_event_unref (stream->pending_newsegment);
     stream->pending_newsegment = NULL;
   }
 
-
   if (stream->downloader != NULL) {
     g_object_unref (stream->downloader);
     stream->downloader = NULL;
@@ -893,6 +892,11 @@ gst_mss_demux_process_manifest (GstMssDemux * mssdemux)
 
     mssdemux->manifest_uri = g_strdup (uri);
     baseurl_end = g_strrstr (uri, "/Manifest");
+    if (baseurl_end == NULL) {
+      /* second try */
+      baseurl_end = g_strrstr (uri, "/manifest");
+    }
+
     if (baseurl_end) {
       /* set the new end of the string */
       baseurl_end[0] = '\0';
@@ -1022,6 +1026,7 @@ gst_mss_demux_stream_store_object (GstMssDemuxStream * stream,
     GstMiniObject * obj)
 {
   GstDataQueueItem *item;
+  gboolean ret = FALSE;
 
   item = g_slice_new (GstDataQueueItem);
   item->object = (GstMiniObject *) obj;
@@ -1032,7 +1037,13 @@ gst_mss_demux_stream_store_object (GstMssDemuxStream * stream,
 
   item->destroy = (GDestroyNotify) _free_data_queue_item;
 
-  if (!gst_data_queue_push (stream->dataqueue, item)) {
+  if (G_LIKELY (GST_IS_BUFFER (obj))) {
+    ret = gst_data_queue_push (stream->dataqueue, item);
+  } else {
+    ret = gst_data_queue_push_force (stream->dataqueue, item);
+  }
+
+  if (!ret) {
     GST_DEBUG_OBJECT (stream->parent, "Failed to store object %p", obj);
     item->destroy (item);
   }
