@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -233,7 +233,7 @@ gst_buffer_add_wayland_meta (GstBuffer * buffer, GstWaylandBufferPool * wpool)
     return NULL;
 
   wmeta->wbuffer = wl_shm_pool_create_buffer (sink->shm_pool->pool, offset,
-      sink->video_width, sink->video_height, stride, WL_SHM_FORMAT_XRGB8888);
+      sink->video_width, sink->video_height, stride, sink->format);
 
   wmeta->data = data;
   wmeta->size = size;
