@@ -23,8 +23,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -93,6 +93,14 @@
 GST_DEBUG_CATEGORY (h264_parser_debug);
 #define GST_CAT_DEFAULT h264_parser_debug
 
+static gboolean initialized = FALSE;
+#define INITIALIZE_DEBUG_CATEGORY \
+  if (!initialized) { \
+    GST_DEBUG_CATEGORY_INIT (h264_parser_debug, "codecparsers_h264", 0, \
+        "h264 parser library"); \
+    initialized = TRUE; \
+  }
+
 /**** Default scaling_lists according to Table 7-2 *****/
 static const guint8 default_4x4_intra[16] = {
   6, 13, 13, 20, 20, 20, 28, 28, 28, 28, 32, 32,
@@ -497,6 +505,14 @@ scan_for_start_codes (const guint8 * data, guint size)
       0, size);
 }
 
+static gboolean
+gst_h264_parser_byte_aligned (NalReader * nr)
+{
+  if (nr->bits_in_cache != 0)
+    return FALSE;
+  return TRUE;
+}
+
 static gboolean
 gst_h264_parser_more_data (NalReader * nr)
 {
@@ -984,7 +1000,7 @@ error:
   return FALSE;
 }
 
-static gboolean
+static GstH264ParserResult
 gst_h264_parser_parse_buffering_period (GstH264NalParser * nalparser,
     GstH264BufferingPeriod * per, NalReader * nr)
 {
@@ -1096,7 +1112,7 @@ error:
   return FALSE;
 }
 
-static gboolean
+static GstH264ParserResult
 gst_h264_parser_parse_pic_timing (GstH264NalParser * nalparser,
     GstH264PicTiming * tim, NalReader * nr)
 {
@@ -1118,7 +1134,7 @@ gst_h264_parser_parse_pic_timing (GstH264NalParser * nalparser,
           vui->nal_hrd_parameters.cpb_removal_delay_length_minus1 + 1);
       READ_UINT32 (nr, tim->dpb_output_delay,
           vui->nal_hrd_parameters.dpb_output_delay_length_minus1 + 1);
-    } else if (vui->nal_hrd_parameters_present_flag) {
+    } else if (vui->vcl_hrd_parameters_present_flag) {
       READ_UINT32 (nr, tim->cpb_removal_delay,
           vui->vcl_hrd_parameters.cpb_removal_delay_length_minus1 + 1);
       READ_UINT32 (nr, tim->dpb_output_delay,
@@ -1155,6 +1171,78 @@ error:
   return GST_H264_PARSER_ERROR;
 }
 
+static GstH264ParserResult
+gst_h264_parser_parse_sei_message (GstH264NalParser * nalparser,
+    NalReader * nr, GstH264SEIMessage * sei)
+{
+  guint32 payloadSize;
+  guint8 payload_type_byte, payload_size_byte;
+#ifndef GST_DISABLE_GST_DEBUG
+  guint remaining, payload_size;
+#endif
+  GstH264ParserResult res;
+
+  GST_DEBUG ("parsing \"Sei message\"");
+
+  sei->payloadType = 0;
+  do {
+    READ_UINT8 (nr, payload_type_byte, 8);
+    sei->payloadType += payload_type_byte;
+  } while (payload_type_byte == 0xff);
+
+  payloadSize = 0;
+  do {
+    READ_UINT8 (nr, payload_size_byte, 8);
+    payloadSize += payload_size_byte;
+  }
+  while (payload_size_byte == 0xff);
+
+#ifndef GST_DISABLE_GST_DEBUG
+  remaining = nal_reader_get_remaining (nr);
+  payload_size = payloadSize * 8 < remaining ? payloadSize * 8 : remaining;
+
+  GST_DEBUG ("SEI message received: payloadType  %u, payloadSize = %u bits",
+      sei->payloadType, payload_size);
+#endif
+
+  if (sei->payloadType == GST_H264_SEI_BUF_PERIOD) {
+    /* size not set; might depend on emulation_prevention_three_byte */
+    res = gst_h264_parser_parse_buffering_period (nalparser,
+        &sei->payload.buffering_period, nr);
+  } else if (sei->payloadType == GST_H264_SEI_PIC_TIMING) {
+    /* size not set; might depend on emulation_prevention_three_byte */
+    res = gst_h264_parser_parse_pic_timing (nalparser,
+        &sei->payload.pic_timing, nr);
+  } else {
+    /* Just consume payloadSize */
+    guint32 i;
+    for (i = 0; i < payloadSize; i++)
+      nal_reader_skip_to_byte (nr);
+    res = GST_H264_PARSER_OK;
+  }
+
+  /* Check for message that doesn't end at byte boundary */
+  if (!gst_h264_parser_byte_aligned (nr)) {
+    guint8 bit_equal_to_one;
+    READ_UINT8 (nr, bit_equal_to_one, 1);
+    if (!bit_equal_to_one)
+      GST_WARNING ("Bit non equal to one.");
+
+    while (!gst_h264_parser_byte_aligned (nr)) {
+      guint8 bit_equal_to_zero;
+      READ_UINT8 (nr, bit_equal_to_zero, 1);
+      if (bit_equal_to_zero)
+        GST_WARNING ("Bit non equal to zero.");
+    }
+  }
+
+  return res;
+
+error:
+  GST_WARNING ("error parsing \"Sei message\"");
+  return GST_H264_PARSER_ERROR;
+}
+
 /******** API *************/
 
 /**
@@ -1171,8 +1259,7 @@ gst_h264_nal_parser_new (void)
   GstH264NalParser *nalparser;
 
   nalparser = g_slice_new0 (GstH264NalParser);
-  GST_DEBUG_CATEGORY_INIT (h264_parser_debug, "codecparsers_h264", 0,
-      "h264 parser library");
+  INITIALIZE_DEBUG_CATEGORY;
 
   return nalparser;
 }
@@ -1291,7 +1378,10 @@ gst_h264_parser_identify_nalu (GstH264NalParser * nalparser,
     return GST_H264_PARSER_NO_NAL_END;
   }
 
-  if (off2 > 0 && data[nalu->offset + off2 - 1] == 00)
+  /* Mini performance improvement:
+   * We could have a way to store how many 0s were skipped to avoid
+   * parsing them again on the next NAL */
+  while (off2 > 0 && data[nalu->offset + off2 - 1] == 00)
     off2--;
 
   nalu->size = off2;
@@ -1430,11 +1520,11 @@ gst_h264_parse_sps (GstH264NalUnit * nalu, GstH264SPS * sps,
 {
   NalReader nr;
   gint width, height;
-  guint8 frame_cropping_flag;
   guint subwc[] = { 1, 2, 2, 1 };
   guint subhc[] = { 1, 2, 1, 1 };
   GstH264VUIParams *vui = NULL;
 
+  INITIALIZE_DEBUG_CATEGORY;
   GST_DEBUG ("parsing SPS");
   nal_reader_init (&nr, nalu->data + nalu->offset + 1, nalu->size - 1);
 
@@ -1522,8 +1612,8 @@ gst_h264_parse_sps (GstH264NalUnit * nalu, GstH264SPS * sps,
     READ_UINT8 (&nr, sps->mb_adaptive_frame_field_flag, 1);
 
   READ_UINT8 (&nr, sps->direct_8x8_inference_flag, 1);
-  READ_UINT8 (&nr, frame_cropping_flag, 1);
-  if (frame_cropping_flag) {
+  READ_UINT8 (&nr, sps->frame_cropping_flag, 1);
+  if (sps->frame_cropping_flag) {
     READ_UE (&nr, sps->frame_crop_left_offset);
     READ_UE (&nr, sps->frame_crop_right_offset);
     READ_UE (&nr, sps->frame_crop_top_offset);
@@ -1549,19 +1639,32 @@ gst_h264_parse_sps (GstH264NalUnit * nalu, GstH264SPS * sps,
   height = (sps->pic_height_in_map_units_minus1 + 1);
   height *= 16 * (2 - sps->frame_mbs_only_flag);
   GST_LOG ("initial width=%d, height=%d", width, height);
-
-  width -= (sps->frame_crop_left_offset + sps->frame_crop_right_offset)
-      * subwc[sps->chroma_format_idc];
-  height -= (sps->frame_crop_top_offset + sps->frame_crop_bottom_offset
-      * subhc[sps->chroma_format_idc] * (2 - sps->frame_mbs_only_flag));
   if (width < 0 || height < 0) {
     GST_WARNING ("invalid width/height in SPS");
-    return FALSE;
+    goto error;
   }
-  GST_LOG ("final width=%u, height=%u", width, height);
+
   sps->width = width;
   sps->height = height;
 
+  if (sps->frame_cropping_flag) {
+    const guint crop_unit_x = subwc[sps->chroma_format_idc];
+    const guint crop_unit_y =
+        subhc[sps->chroma_format_idc] * (2 - sps->frame_mbs_only_flag);
+
+    width -= (sps->frame_crop_left_offset + sps->frame_crop_right_offset)
+        * crop_unit_x;
+    height -= (sps->frame_crop_top_offset + sps->frame_crop_bottom_offset)
+        * crop_unit_y;
+
+    sps->crop_rect_width = width;
+    sps->crop_rect_height = height;
+    sps->crop_rect_x = sps->frame_crop_left_offset * crop_unit_x;
+    sps->crop_rect_y = sps->frame_crop_top_offset * crop_unit_y;
+
+    GST_LOG ("crop_rectangle x=%u y=%u width=%u, height=%u", sps->crop_rect_x,
+        sps->crop_rect_y, width, height);
+  }
   sps->fps_num = 0;
   sps->fps_den = 1;
 
@@ -1614,6 +1717,7 @@ gst_h264_parse_pps (GstH264NalParser * nalparser, GstH264NalUnit * nalu,
   guint8 pic_scaling_matrix_present_flag;
   gint qp_bd_offset;
 
+  INITIALIZE_DEBUG_CATEGORY;
   GST_DEBUG ("parsing PPS");
 
   nal_reader_init (&nr, nalu->data + nalu->offset + 1, nalu->size - 1);
@@ -1700,8 +1804,8 @@ gst_h264_parse_pps (GstH264NalParser * nalparser, GstH264NalUnit * nalu,
     if (sps->scaling_matrix_present_flag) {
       if (!gst_h264_parser_parse_scaling_list (&nr,
               pps->scaling_lists_4x4, pps->scaling_lists_8x8,
-              sps->scaling_lists_4x4[0], sps->scaling_lists_4x4[3],
-              sps->scaling_lists_8x8[0], sps->scaling_lists_8x8[3], n_lists))
+              sps->scaling_lists_4x4[3], sps->scaling_lists_4x4[0],
+              sps->scaling_lists_8x8[3], sps->scaling_lists_8x8[0], n_lists))
         goto error;
     } else {
       if (!gst_h264_parser_parse_scaling_list (&nr,
@@ -1933,63 +2037,31 @@ error:
  * gst_h264_parser_parse_sei:
  * @nalparser: a #GstH264NalParser
  * @nalu: The #GST_H264_NAL_SEI #GstH264NalUnit to parse
- * @sei: The #GstH264SEIMessage to fill.
+ * @messages: The GArray of #GstH264SEIMessage to fill. The caller must free it when done.
  *
- * Parses @data, and fills the @sei structures.
+ * Parses @data, create and fills the @messages array.
  *
  * Returns: a #GstH264ParserResult
  */
 GstH264ParserResult
 gst_h264_parser_parse_sei (GstH264NalParser * nalparser, GstH264NalUnit * nalu,
-    GstH264SEIMessage * sei)
+    GArray ** messages)
 {
   NalReader nr;
+  GstH264SEIMessage sei;
+  GstH264ParserResult res;
 
-  guint32 payloadSize;
-  guint8 payload_type_byte, payload_size_byte;
-  guint remaining, payload_size;
-  gboolean res;
-
-  GST_DEBUG ("parsing \"Sei message\"");
-
+  GST_DEBUG ("parsing SEI nal");
   nal_reader_init (&nr, nalu->data + nalu->offset + 1, nalu->size - 1);
+  *messages = g_array_new (FALSE, FALSE, sizeof (GstH264SEIMessage));
 
-  /* init */
-  memset (sei, 0, sizeof (*sei));
-
-  sei->payloadType = 0;
   do {
-    READ_UINT8 (&nr, payload_type_byte, 8);
-    sei->payloadType += payload_type_byte;
-  } while (payload_type_byte == 0xff);
-
-  payloadSize = 0;
-  do {
-    READ_UINT8 (&nr, payload_size_byte, 8);
-    payloadSize += payload_size_byte;
-  }
-  while (payload_size_byte == 0xff);
-
-  remaining = nal_reader_get_remaining (&nr) * 8;
-  payload_size = payloadSize < remaining ? payloadSize : remaining;
-
-  GST_DEBUG ("SEI message received: payloadType  %u, payloadSize = %u bytes",
-      sei->payloadType, payload_size);
-
-  if (sei->payloadType == GST_H264_SEI_BUF_PERIOD) {
-    /* size not set; might depend on emulation_prevention_three_byte */
-    res = gst_h264_parser_parse_buffering_period (nalparser,
-        &sei->payload.buffering_period, &nr);
-  } else if (sei->payloadType == GST_H264_SEI_PIC_TIMING) {
-    /* size not set; might depend on emulation_prevention_three_byte */
-    res = gst_h264_parser_parse_pic_timing (nalparser,
-        &sei->payload.pic_timing, &nr);
-  } else
-    res = GST_H264_PARSER_OK;
+    res = gst_h264_parser_parse_sei_message (nalparser, &nr, &sei);
+    if (res == GST_H264_PARSER_OK)
+      g_array_append_val (*messages, sei);
+    else
+      break;
+  } while (gst_h264_parser_more_data (&nr));
 
   return res;
-
-error:
-  GST_WARNING ("error parsing \"Sei message\"");
-  return GST_H264_PARSER_ERROR;
 }
