@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* based on:
@@ -376,9 +376,13 @@ static GstFlowReturn
 gst_asf_mux_push_buffer (GstAsfMux * asfmux, GstBuffer * buf)
 {
   GstFlowReturn ret;
+  gsize bufsize = gst_buffer_get_size (buf);
+
   ret = gst_pad_push (asfmux->srcpad, buf);
+
   if (ret == GST_FLOW_OK)
-    asfmux->file_size += gst_buffer_get_size (buf);
+    asfmux->file_size += bufsize;
+
   return ret;
 }
 
@@ -735,11 +739,11 @@ gst_asf_mux_write_stream_properties (GstAsfMux * asfmux, guint8 ** buf,
         "wave formatex values: codec_id=%" G_GUINT16_FORMAT ", channels=%"
         G_GUINT16_FORMAT ", rate=%" G_GUINT32_FORMAT ", bytes_per_sec=%"
         G_GUINT32_FORMAT ", block_alignment=%" G_GUINT16_FORMAT
-        ", bits_per_sample=%" G_GUINT16_FORMAT ", codec_data_length=%"
-        G_GUINT16_FORMAT, audiopad->audioinfo.format,
-        audiopad->audioinfo.channels, audiopad->audioinfo.rate,
-        audiopad->audioinfo.av_bps, audiopad->audioinfo.blockalign,
-        audiopad->audioinfo.bits_per_sample, codec_data_length);
+        ", bits_per_sample=%" G_GUINT16_FORMAT ", codec_data_length=%u",
+        audiopad->audioinfo.format, audiopad->audioinfo.channels,
+        audiopad->audioinfo.rate, audiopad->audioinfo.av_bps,
+        audiopad->audioinfo.blockalign, audiopad->audioinfo.bits_per_sample,
+        codec_data_length);
 
 
     *buf += ASF_AUDIO_SPECIFIC_DATA_SIZE;
@@ -1260,6 +1264,7 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
   guint64 padding = asfmux->prop_padding;
   GstSegment segment;
   GstMapInfo map;
+  gchar s_id[32];
 
   if (padding < ASF_PADDING_OBJECT_SIZE)
     padding = 0;
@@ -1268,6 +1273,10 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
   GST_INFO_OBJECT (asfmux, "Writing headers");
   asfmux->state = GST_ASF_MUX_STATE_HEADERS;
 
+  /* stream-start (FIXME: create id based on input ids) */
+  g_snprintf (s_id, sizeof (s_id), "asfmux-%08x", g_random_int ());
+  gst_pad_push_event (asfmux->srcpad, gst_event_new_stream_start (s_id));
+
   caps = gst_pad_get_pad_template_caps (asfmux->srcpad);
   gst_pad_set_caps (asfmux->srcpad, caps);
   gst_caps_unref (caps);
@@ -1486,7 +1495,7 @@ gst_asf_mux_flush_payloads (GstAsfMux * asfmux)
     GST_DEBUG_OBJECT (asfmux, "stream number: %d", pad->stream_number & 0x7F);
     GST_DEBUG_OBJECT (asfmux, "media object number: %d",
         (gint) payload->media_obj_num);
-    GST_DEBUG_OBJECT (asfmux, "offset into media object: %" G_GUINT16_FORMAT,
+    GST_DEBUG_OBJECT (asfmux, "offset into media object: %" G_GUINT32_FORMAT,
         payload->offset_in_media_obj);
     GST_DEBUG_OBJECT (asfmux, "media object size: %" G_GUINT32_FORMAT,
         payload->media_object_size);
@@ -1675,7 +1684,7 @@ gst_asf_mux_push_simple_index (GstAsfMux * asfmux, GstAsfVideoPad * pad)
   GST_DEBUG_OBJECT (asfmux,
       "Simple index object values - size:%" G_GUINT64_FORMAT ", time interval:%"
       G_GUINT64_FORMAT ", max packet count:%" G_GUINT32_FORMAT ", entries:%"
-      G_GUINT16_FORMAT, object_size, pad->time_interval,
+      G_GUINT32_FORMAT, object_size, pad->time_interval,
       pad->max_keyframe_packet_count, entries_count);
 
   for (walk = pad->simple_index; walk; walk = g_slist_next (walk)) {
@@ -1853,8 +1862,7 @@ gst_asf_mux_process_buffer (GstAsfMux * asfmux, GstAsfPad * pad,
   payload->pad = (GstCollectData *) pad;
   payload->data = buf;
 
-  GST_LOG_OBJECT (asfmux,
-      "Processing payload data for stream number %" G_GUINT16_FORMAT,
+  GST_LOG_OBJECT (asfmux, "Processing payload data for stream number %u",
       pad->stream_number);
 
   /* stream number */
@@ -1955,8 +1963,9 @@ gst_asf_mux_collected (GstCollectPads * collect, gpointer data)
     /* check the ts for getting the first time */
     if (!GST_CLOCK_TIME_IS_VALID (pad->first_ts) &&
         GST_CLOCK_TIME_IS_VALID (time)) {
-      GST_DEBUG_OBJECT (asfmux, "First ts for stream number %" G_GUINT16_FORMAT
-          ": %" GST_TIME_FORMAT, pad->stream_number, GST_TIME_ARGS (time));
+      GST_DEBUG_OBJECT (asfmux,
+          "First ts for stream number %u: %" GST_TIME_FORMAT,
+          pad->stream_number, GST_TIME_ARGS (time));
       pad->first_ts = time;
       if (!GST_CLOCK_TIME_IS_VALID (asfmux->first_ts) ||
           time < asfmux->first_ts) {
