@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -133,7 +133,7 @@ gst_debug_spy_class_init (GstDebugSpyClass * klass)
           "Checksum algorithm to use", GST_DEBUG_SPY_CHECKSUM_TYPE,
           G_CHECKSUM_SHA1, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_set_metadata (element_class,
+  gst_element_class_set_static_metadata (element_class,
       "DebugSpy",
       "Filter/Analyzer/Debug",
       "DebugSpy provides information on buffers with bus messages",
