@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,8 +26,6 @@
 #include "gstfrei0r.h"
 #include "gstfrei0rfilter.h"
 
-#include <gst/controller/gstcontroller.h>
-
 GST_DEBUG_CATEGORY_EXTERN (frei0r_debug);
 #define GST_CAT_DEFAULT frei0r_debug
 
@@ -42,11 +40,15 @@ gst_frei0r_filter_set_caps (GstBaseTransform * trans, GstCaps * incaps,
     GstCaps * outcaps)
 {
   GstFrei0rFilter *self = GST_FREI0R_FILTER (trans);
-  GstVideoFormat fmt;
+  GstVideoInfo info;
 
-  if (!gst_video_format_parse_caps (incaps, &fmt, &self->width, &self->height))
+  gst_video_info_init (&info);
+  if (!gst_video_info_from_caps (&info, incaps))
     return FALSE;
 
+  self->width = info.width;
+  self->height = info.height;
+
   return TRUE;
 }
 
@@ -91,6 +93,7 @@ gst_frei0r_filter_transform (GstBaseTransform * trans, GstBuffer * inbuf,
   GstFrei0rFilter *self = GST_FREI0R_FILTER (trans);
   GstFrei0rFilterClass *klass = GST_FREI0R_FILTER_GET_CLASS (trans);
   gdouble time;
+  GstMapInfo inmap, outmap;
 
   if (G_UNLIKELY (self->width <= 0 || self->height <= 0))
     return GST_FLOW_NOT_NEGOTIATED;
@@ -106,14 +109,20 @@ gst_frei0r_filter_transform (GstBaseTransform * trans, GstBuffer * inbuf,
   time = ((gdouble) GST_BUFFER_TIMESTAMP (inbuf)) / GST_SECOND;
 
   GST_OBJECT_LOCK (self);
+
+  gst_buffer_map (inbuf, &inmap, GST_MAP_READ);
+  gst_buffer_map (outbuf, &outmap, GST_MAP_WRITE);
+
   if (klass->ftable->update2)
     klass->ftable->update2 (self->f0r_instance, time,
-        (const guint32 *) GST_BUFFER_DATA (inbuf), NULL, NULL,
-        (guint32 *) GST_BUFFER_DATA (outbuf));
+        (const guint32 *) inmap.data, NULL, NULL, (guint32 *) outmap.data);
   else
     klass->ftable->update (self->f0r_instance, time,
-        (const guint32 *) GST_BUFFER_DATA (inbuf),
-        (guint32 *) GST_BUFFER_DATA (outbuf));
+        (const guint32 *) inmap.data, (guint32 *) outmap.data);
+
+  gst_buffer_unmap (outbuf, &outmap);
+  gst_buffer_unmap (inbuf, &inmap);
+
   GST_OBJECT_UNLOCK (self);
 
   return GST_FLOW_OK;
@@ -176,6 +185,7 @@ gst_frei0r_filter_class_init (GstFrei0rFilterClass * klass,
   GstElementClass *gstelement_class = (GstElementClass *) klass;
   GstBaseTransformClass *gsttrans_class = (GstBaseTransformClass *) klass;
   GstPadTemplate *templ;
+  const gchar *desc;
   GstCaps *caps;
   gchar *author;
 
@@ -196,8 +206,11 @@ gst_frei0r_filter_class_init (GstFrei0rFilterClass * klass,
       g_strdup_printf
       ("Sebastian Dröge <sebastian.droege@collabora.co.uk>, %s",
       class_data->info.author);
+  desc = class_data->info.explanation;
+  if (desc == NULL || *desc == '\0')
+    desc = "No details";
   gst_element_class_set_metadata (gstelement_class, class_data->info.name,
-      "Filter/Effect/Video", class_data->info.explanation, author);
+      "Filter/Effect/Video", desc, author);
   g_free (author);
 
   caps = gst_frei0r_caps_from_color_model (class_data->info.color_model);
