@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -1357,7 +1357,7 @@ gst_mve_mux_base_init (GstMveMuxClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&video_sink_factory));
 
-  gst_element_class_set_metadata (element_class, "MVE Multiplexer",
+  gst_element_class_set_static_metadata (element_class, "MVE Multiplexer",
       "Codec/Muxer",
       "Muxes audio and video into an MVE stream",
       "Jens Granseuer <jensgr@gmx.net>");
