@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-irtspparse
@@ -110,7 +110,7 @@ gst_irtsp_parse_class_init (GstIRTSPParseClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_metadata (element_class, "IRTSPParse",
+  gst_element_class_set_static_metadata (element_class, "IRTSPParse",
       "Raw/Parser",
       "Parses a raw interleaved RTSP stream",
       "Mark Nauwelaerts <mark.nauwelaerts@collabora.co.uk>");
