@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-audioparse
@@ -70,7 +70,8 @@ enum
   PROP_RATE,
   PROP_CHANNELS,
   PROP_INTERLEAVED,
-  PROP_CHANNEL_POSITIONS
+  PROP_CHANNEL_POSITIONS,
+  PROP_USE_SINK_CAPS
 };
 
 #define GST_AUDIO_PARSE_FORMAT (gst_audio_parse_format_get_type ())
@@ -146,7 +147,12 @@ gst_audio_parse_class_init (GstAudioParseClass * klass)
               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS),
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_set_metadata (gstelement_class, "Audio Parse",
+  g_object_class_install_property (gobject_class, PROP_USE_SINK_CAPS,
+      g_param_spec_boolean ("use-sink-caps", "Use sink caps",
+          "Use the sink caps for the format, only performing timestamping",
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_element_class_set_static_metadata (gstelement_class, "Audio Parse",
       "Filter/Audio",
       "Converts stream into audio frames",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
@@ -206,6 +212,9 @@ gst_audio_parse_set_property (GObject * object, guint prop_id,
 
       ap->channel_positions = g_value_dup_boxed (value);
       break;
+    case PROP_USE_SINK_CAPS:
+      ap->use_sink_caps = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -243,6 +252,9 @@ gst_audio_parse_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_CHANNEL_POSITIONS:
       g_value_set_boxed (value, ap->channel_positions);
       break;
+    case PROP_USE_SINK_CAPS:
+      g_value_set_boolean (value, ap->use_sink_caps);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -356,6 +368,23 @@ gst_audio_parse_get_caps (GstRawParse * rp)
   gint fps_n, fps_d;
   const GValue *val;
 
+  if (ap->use_sink_caps) {
+    gint rate;
+    GstCaps *caps = gst_pad_get_current_caps (rp->sinkpad);
+    gst_audio_info_from_caps (&info, caps);
+
+    ap->format = GST_AUDIO_PARSE_FORMAT_RAW;
+    ap->raw_format = GST_AUDIO_INFO_FORMAT (&info);
+    ap->channels = GST_AUDIO_INFO_CHANNELS (&info);
+    ap->interleaved = info.layout == GST_AUDIO_LAYOUT_INTERLEAVED;
+
+    rate = GST_AUDIO_INFO_RATE (&info);
+    gst_raw_parse_set_fps (GST_RAW_PARSE (ap), rate, 1);
+    gst_audio_parse_update_frame_size (ap);
+
+    return caps;
+  }
+
   gst_raw_parse_get_fps (rp, &fps_n, &fps_d);
   gst_audio_parse_setup_channel_positions (ap);
 
