@@ -50,7 +50,8 @@ enum
 {
   GST_H264_PARSE_FORMAT_NONE,
   GST_H264_PARSE_FORMAT_AVC,
-  GST_H264_PARSE_FORMAT_BYTE
+  GST_H264_PARSE_FORMAT_BYTE,
+  GST_H264_PARSE_FORMAT_AVC3
 };
 
 enum
@@ -69,7 +70,7 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-h264, parsed = (boolean) true, "
-        "stream-format=(string) { avc, byte-stream }, "
+        "stream-format=(string) { avc, avc3, byte-stream }, "
         "alignment=(string) { au, nal }"));
 
 #define parent_class gst_h264_parse_parent_class
@@ -261,6 +262,8 @@ gst_h264_parse_get_string (GstH264Parse * parse, gboolean format, gint code)
         return "avc";
       case GST_H264_PARSE_FORMAT_BYTE:
         return "byte-stream";
+      case GST_H264_PARSE_FORMAT_AVC3:
+        return "avc3";
       default:
         return "none";
     }
@@ -299,6 +302,8 @@ gst_h264_parse_format_from_caps (GstCaps * caps, guint * format, guint * align)
           *format = GST_H264_PARSE_FORMAT_AVC;
         else if (strcmp (str, "byte-stream") == 0)
           *format = GST_H264_PARSE_FORMAT_BYTE;
+        else if (strcmp (str, "avc3") == 0)
+          *format = GST_H264_PARSE_FORMAT_AVC3;
       }
     }
 
@@ -378,7 +383,8 @@ gst_h264_parse_wrap_nal (GstH264Parse * h264parse, guint format, guint8 * data,
   GST_DEBUG_OBJECT (h264parse, "nal length %d", size);
 
   buf = gst_buffer_new_allocate (NULL, 4 + size, NULL);
-  if (format == GST_H264_PARSE_FORMAT_AVC) {
+  if (format == GST_H264_PARSE_FORMAT_AVC
+      || format == GST_H264_PARSE_FORMAT_AVC3) {
     tmp = GUINT32_TO_BE (size << (32 - 8 * nl));
   } else {
     /* HACK: nl should always be 4 here, otherwise this won't work. 
@@ -453,6 +459,50 @@ _nal_name (GstH264NalUnitType nal_type)
 }
 #endif
 
+static void
+gst_h264_parse_process_sei (GstH264Parse * h264parse, GstH264NalUnit * nalu)
+{
+  GstH264SEIMessage sei;
+  GstH264NalParser *nalparser = h264parse->nalparser;
+  GstH264ParserResult pres;
+  GArray *messages;
+  guint i;
+
+  pres = gst_h264_parser_parse_sei (nalparser, nalu, &messages);
+  if (pres != GST_H264_PARSER_OK)
+    GST_WARNING_OBJECT (h264parse, "failed to parse one ore more SEI message");
+
+  /* Even if pres != GST_H264_PARSER_OK, some message could have been parsed and
+   * stored in messages.
+   */
+  for (i = 0; i < messages->len; i++) {
+    sei = g_array_index (messages, GstH264SEIMessage, i);
+    switch (sei.payloadType) {
+      case GST_H264_SEI_PIC_TIMING:
+        h264parse->sei_pic_struct_pres_flag =
+            sei.payload.pic_timing.pic_struct_present_flag;
+        h264parse->sei_cpb_removal_delay =
+            sei.payload.pic_timing.cpb_removal_delay;
+        if (h264parse->sei_pic_struct_pres_flag)
+          h264parse->sei_pic_struct = sei.payload.pic_timing.pic_struct;
+        GST_LOG_OBJECT (h264parse, "pic timing updated");
+        break;
+      case GST_H264_SEI_BUF_PERIOD:
+        if (h264parse->ts_trn_nb == GST_CLOCK_TIME_NONE ||
+            h264parse->dts == GST_CLOCK_TIME_NONE)
+          h264parse->ts_trn_nb = 0;
+        else
+          h264parse->ts_trn_nb = h264parse->dts;
+
+        GST_LOG_OBJECT (h264parse,
+            "new buffering period; ts_trn_nb updated: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (h264parse->ts_trn_nb));
+        break;
+    }
+  }
+  g_array_free (messages, TRUE);
+}
+
 /* caller guarantees 2 bytes of nal payload */
 static void
 gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
@@ -460,7 +510,6 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
   guint nal_type;
   GstH264PPS pps = { 0, };
   GstH264SPS sps = { 0, };
-  GstH264SEIMessage sei;
   GstH264NalParser *nalparser = h264parse->nalparser;
   GstH264ParserResult pres;
 
@@ -521,28 +570,7 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
       gst_h264_parser_store_nal (h264parse, pps.id, nal_type, nalu);
       break;
     case GST_H264_NAL_SEI:
-      gst_h264_parser_parse_sei (nalparser, nalu, &sei);
-      switch (sei.payloadType) {
-        case GST_H264_SEI_PIC_TIMING:
-          h264parse->sei_pic_struct_pres_flag =
-              sei.payload.pic_timing.pic_struct_present_flag;
-          h264parse->sei_cpb_removal_delay =
-              sei.payload.pic_timing.cpb_removal_delay;
-          if (h264parse->sei_pic_struct_pres_flag)
-            h264parse->sei_pic_struct = sei.payload.pic_timing.pic_struct;
-          break;
-        case GST_H264_SEI_BUF_PERIOD:
-          if (h264parse->ts_trn_nb == GST_CLOCK_TIME_NONE ||
-              h264parse->dts == GST_CLOCK_TIME_NONE)
-            h264parse->ts_trn_nb = 0;
-          else
-            h264parse->ts_trn_nb = h264parse->dts;
-
-          GST_LOG_OBJECT (h264parse,
-              "new buffering period; ts_trn_nb updated: %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (h264parse->ts_trn_nb));
-          break;
-      }
+      gst_h264_parse_process_sei (h264parse, nalu);
       /* mark SEI pos */
       if (h264parse->sei_pos == -1) {
         if (h264parse->transform)
@@ -576,16 +604,8 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
             "parse result %d, first MB: %u, slice type: %u",
             pres, slice.first_mb_in_slice, slice.type);
         if (pres == GST_H264_PARSER_OK) {
-          switch (slice.type) {
-            case 2:
-            case 4:
-            case 7:
-            case 9:
-              h264parse->keyframe |= TRUE;
-
-            default:
-              break;
-          }
+          if (GST_H264_IS_I_SLICE (&slice) || GST_H264_IS_SI_SLICE (&slice))
+            h264parse->keyframe |= TRUE;
         }
       }
       if (G_LIKELY (nal_type != GST_H264_NAL_SLICE_IDR &&
@@ -717,12 +737,21 @@ gst_h264_parse_handle_frame_packetized (GstBaseParse * parse,
 
     /* dispatch per NALU if needed */
     if (h264parse->split_packetized) {
+      GstBaseParseFrame tmp_frame;
+
+      gst_base_parse_frame_init (&tmp_frame);
+      tmp_frame.flags |= frame->flags;
+      tmp_frame.offset = frame->offset;
+      tmp_frame.overhead = frame->overhead;
+      tmp_frame.buffer = gst_buffer_copy_region (buffer, GST_BUFFER_COPY_ALL,
+          nalu.offset, nalu.size);
+
       /* note we don't need to come up with a sub-buffer, since
        * subsequent code only considers input buffer's metadata.
        * Real data is either taken from input by baseclass or
        * a replacement output buffer is provided anyway. */
-      gst_h264_parse_parse_frame (parse, frame);
-      ret = gst_base_parse_finish_frame (parse, frame, nl + nalu.size);
+      gst_h264_parse_parse_frame (parse, &tmp_frame);
+      ret = gst_base_parse_finish_frame (parse, &tmp_frame, nl + nalu.size);
       left -= nl + nalu.size;
     }
 
@@ -1010,10 +1039,17 @@ gst_h264_parse_make_codec_data (GstH264Parse * h264parse)
     }
   }
 
+  /* AVC3 has SPS/PPS inside the stream, not in the codec_data */
+  if (h264parse->format == GST_H264_PARSE_FORMAT_AVC3) {
+    num_sps = sps_size = 0;
+    num_pps = pps_size = 0;
+  }
+
   GST_DEBUG_OBJECT (h264parse,
       "constructing codec_data: num_sps=%d, num_pps=%d", num_sps, num_pps);
 
-  if (!found || !num_pps)
+  if (!found || (0 == num_pps
+          && GST_H264_PARSE_FORMAT_AVC3 != h264parse->format))
     return NULL;
 
   buf = gst_buffer_new_allocate (NULL, 5 + 1 + sps_size + 1 + pps_size, NULL);
@@ -1029,23 +1065,27 @@ gst_h264_parse_make_codec_data (GstH264Parse * h264parse)
   data[5] = 0xe0 | num_sps;     /* number of SPSs */
 
   data += 6;
-  for (i = 0; i < GST_H264_MAX_SPS_COUNT; i++) {
-    if ((nal = h264parse->sps_nals[i])) {
-      gsize nal_size = gst_buffer_get_size (nal);
-      GST_WRITE_UINT16_BE (data, nal_size);
-      gst_buffer_extract (nal, 0, data + 2, nal_size);
-      data += 2 + nal_size;
+  if (h264parse->format != GST_H264_PARSE_FORMAT_AVC3) {
+    for (i = 0; i < GST_H264_MAX_SPS_COUNT; i++) {
+      if ((nal = h264parse->sps_nals[i])) {
+        gsize nal_size = gst_buffer_get_size (nal);
+        GST_WRITE_UINT16_BE (data, nal_size);
+        gst_buffer_extract (nal, 0, data + 2, nal_size);
+        data += 2 + nal_size;
+      }
     }
   }
 
   data[0] = num_pps;
   data++;
-  for (i = 0; i < GST_H264_MAX_PPS_COUNT; i++) {
-    if ((nal = h264parse->pps_nals[i])) {
-      gsize nal_size = gst_buffer_get_size (nal);
-      GST_WRITE_UINT16_BE (data, nal_size);
-      gst_buffer_extract (nal, 0, data + 2, nal_size);
-      data += 2 + nal_size;
+  if (h264parse->format != GST_H264_PARSE_FORMAT_AVC3) {
+    for (i = 0; i < GST_H264_MAX_PPS_COUNT; i++) {
+      if ((nal = h264parse->pps_nals[i])) {
+        gsize nal_size = gst_buffer_get_size (nal);
+        GST_WRITE_UINT16_BE (data, nal_size);
+        gst_buffer_extract (nal, 0, data + 2, nal_size);
+        data += 2 + nal_size;
+      }
     }
   }
 
@@ -1178,8 +1218,9 @@ gst_h264_parse_update_src_caps (GstH264Parse * h264parse, GstCaps * caps)
   GST_DEBUG_OBJECT (h264parse, "sps: %p", sps);
 
   /* only codec-data for nice-and-clean au aligned packetized avc format */
-  if (h264parse->format == GST_H264_PARSE_FORMAT_AVC &&
-      h264parse->align == GST_H264_PARSE_ALIGN_AU) {
+  if ((h264parse->format == GST_H264_PARSE_FORMAT_AVC
+          || h264parse->format == GST_H264_PARSE_FORMAT_AVC3)
+      && h264parse->align == GST_H264_PARSE_ALIGN_AU) {
     buf = gst_h264_parse_make_codec_data (h264parse);
     if (buf && h264parse->codec_data) {
       GstMapInfo map;
@@ -1311,22 +1352,35 @@ gst_h264_parse_update_src_caps (GstH264Parse * h264parse, GstCaps * caps)
           par_n, par_d, NULL);
     }
 
-    if (buf) {
-      gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER, buf, NULL);
-      gst_buffer_replace (&h264parse->codec_data, buf);
-      gst_buffer_unref (buf);
-      buf = NULL;
-    } else {
-      GstStructure *s;
-      /* remove any left-over codec-data hanging around */
-      s = gst_caps_get_structure (caps, 0);
-      gst_structure_remove_field (s, "codec_data");
-      gst_buffer_replace (&h264parse->codec_data, NULL);
+    src_caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (h264parse));
+
+    if (src_caps
+        && gst_structure_has_field (gst_caps_get_structure (src_caps, 0),
+            "codec_data")) {
+      /* use codec data from old caps for comparison; we don't want to resend caps
+         if everything is same except codec data; */
+      gst_caps_set_value (caps, "codec_data",
+          gst_structure_get_value (gst_caps_get_structure (src_caps, 0),
+              "codec_data"));
     }
 
-    src_caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (h264parse));
-    if (!(src_caps && gst_caps_is_strictly_equal (src_caps, caps)))
+    if (!(src_caps && gst_caps_is_strictly_equal (src_caps, caps))) {
+      /* update codec data to new value */
+      if (buf) {
+        gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER, buf, NULL);
+        gst_buffer_replace (&h264parse->codec_data, buf);
+        gst_buffer_unref (buf);
+        buf = NULL;
+      } else {
+        GstStructure *s;
+        /* remove any left-over codec-data hanging around */
+        s = gst_caps_get_structure (caps, 0);
+        gst_structure_remove_field (s, "codec_data");
+        gst_buffer_replace (&h264parse->codec_data, NULL);
+      }
+
       gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (h264parse), caps);
+    }
 
     if (src_caps)
       gst_caps_unref (src_caps);
@@ -1816,7 +1870,7 @@ gst_h264_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
     size = map.size;
 
     /* parse the avcC data */
-    if (size < 8) {
+    if (size < 7) {             /* when numSPS==0 and numPPS==0, length is 7 bytes */
       gst_buffer_unmap (codec_data, &map);
       goto avcc_too_small;
     }
@@ -1915,7 +1969,8 @@ gst_h264_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
       /* we did parse codec-data and might supplement src caps */
       gst_h264_parse_update_src_caps (h264parse, caps);
     }
-  } else if (format == GST_H264_PARSE_FORMAT_AVC) {
+  } else if (format == GST_H264_PARSE_FORMAT_AVC
+      || format == GST_H264_PARSE_FORMAT_AVC3) {
     /* if input != output, and input is avc, must split before anything else */
     /* arrange to insert codec-data in-stream if needed.
      * src caps are only arranged for later on */
