@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Ole André Vadla Ravnås <oravnas@cisco.com>
+ * Copyright (C) 2010 Ole André Vadla Ravnås <oleavr@soundrop.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "corevideobuffer.h"
@@ -22,15 +22,12 @@
 static void
 gst_core_video_meta_free (GstCoreVideoMeta * meta, GstBuffer * buf)
 {
-  GstCVApi *cv = meta->ctx->cv;
-
   if (meta->pixbuf != NULL) {
-    cv->CVPixelBufferUnlockBaseAddress (meta->pixbuf,
+    CVPixelBufferUnlockBaseAddress (meta->pixbuf,
         kCVPixelBufferLock_ReadOnly);
   }
 
-  cv->CVBufferRelease (meta->cvbuf);
-  g_object_unref (meta->ctx);
+  CVBufferRelease (meta->cvbuf);
 }
 
 GType
@@ -51,21 +48,21 @@ gst_core_video_meta_get_info (void)
 {
   static const GstMetaInfo *core_video_meta_info = NULL;
 
-  if (core_video_meta_info == NULL) {
-    core_video_meta_info = gst_meta_register (GST_CORE_VIDEO_META_API_TYPE,
+  if (g_once_init_enter (&core_video_meta_info)) {
+    const GstMetaInfo *meta = gst_meta_register (GST_CORE_VIDEO_META_API_TYPE,
         "GstCoreVideoMeta", sizeof (GstCoreVideoMeta),
         (GstMetaInitFunction) NULL,
         (GstMetaFreeFunction) gst_core_video_meta_free,
         (GstMetaTransformFunction) NULL);
+    g_once_init_leave (&core_video_meta_info, meta);
   }
   return core_video_meta_info;
 }
 
 GstBuffer *
-gst_core_video_buffer_new (GstCoreMediaCtx * ctx, CVBufferRef cvbuf,
+gst_core_video_buffer_new (CVBufferRef cvbuf,
     GstVideoInfo * vinfo)
 {
-  GstCVApi *cv = ctx->cv;
   void *data;
   size_t size;
   CVPixelBufferRef pixbuf = NULL;
@@ -75,13 +72,13 @@ gst_core_video_buffer_new (GstCoreMediaCtx * ctx, CVBufferRef cvbuf,
   gsize offset[GST_VIDEO_MAX_PLANES];
   gint stride[GST_VIDEO_MAX_PLANES];
 
-  if (CFGetTypeID (cvbuf) != cv->CVPixelBufferGetTypeID ())
+  if (CFGetTypeID (cvbuf) != CVPixelBufferGetTypeID ())
     /* TODO: Do we need to handle other buffer types? */
     goto error;
 
   pixbuf = (CVPixelBufferRef) cvbuf;
 
-  if (cv->CVPixelBufferLockBaseAddress (pixbuf,
+  if (CVPixelBufferLockBaseAddress (pixbuf,
           kCVPixelBufferLock_ReadOnly) != kCVReturnSuccess) {
     goto error;
   }
@@ -91,22 +88,21 @@ gst_core_video_buffer_new (GstCoreMediaCtx * ctx, CVBufferRef cvbuf,
   /* add the corevideo meta to free the underlying corevideo buffer */
   meta = (GstCoreVideoMeta *) gst_buffer_add_meta (buf,
       gst_core_video_meta_get_info (), NULL);
-  meta->ctx = g_object_ref (ctx);
-  meta->cvbuf = cv->CVBufferRetain (cvbuf);
+  meta->cvbuf = CVBufferRetain (cvbuf);
   meta->pixbuf = pixbuf;
 
   /* set stride, offset and size */
   memset (&offset, 0, sizeof (offset));
   memset (&stride, 0, sizeof (stride));
 
-  data = cv->CVPixelBufferGetBaseAddress (pixbuf);
-  height = cv->CVPixelBufferGetHeight (pixbuf);
-  if (cv->CVPixelBufferIsPlanar (pixbuf)) {
+  data = CVPixelBufferGetBaseAddress (pixbuf);
+  height = CVPixelBufferGetHeight (pixbuf);
+  if (CVPixelBufferIsPlanar (pixbuf)) {
     GstVideoInfo tmp_vinfo;
 
-    n_planes = cv->CVPixelBufferGetPlaneCount (pixbuf);
+    n_planes = CVPixelBufferGetPlaneCount (pixbuf);
     for (i = 0; i < n_planes; ++i)
-      stride[i] = cv->CVPixelBufferGetBytesPerRowOfPlane (pixbuf, i);
+      stride[i] = CVPixelBufferGetBytesPerRowOfPlane (pixbuf, i);
 
     /* FIXME: don't hardcode NV12 */
     gst_video_info_init (&tmp_vinfo);
@@ -116,7 +112,7 @@ gst_core_video_buffer_new (GstCoreMediaCtx * ctx, CVBufferRef cvbuf,
     size = tmp_vinfo.size;
   } else {
     n_planes = 1;
-    size = cv->CVPixelBufferGetBytesPerRow (pixbuf) * height;
+    size = CVPixelBufferGetBytesPerRow (pixbuf) * height;
   }
 
   gst_buffer_append_memory (buf,
