@@ -172,6 +172,7 @@ gst_d3dvideosink_init (GstD3DVideoSink * sink)
   sink->create_internal_window = DEFAULT_CREATE_RENDER_WINDOW;
   sink->stream_stop_on_close = DEFAULT_STREAM_STOP_ON_CLOSE;
   sink->enable_navigation_events = DEFAULT_ENABLE_NAVIGATION_EVENTS;
+  sink->d3d.surface = NULL;
 
   g_rec_mutex_init (&sink->lock);
 }
@@ -519,7 +520,9 @@ gst_d3dvideosink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
 
   if (pool) {
     /* we need at least 2 buffer because we hold on to the last one */
+#ifndef DISABLE_BUFFER_POOL
     gst_query_add_allocation_pool (query, pool, size, 2, 0);
+#endif
     gst_object_unref (pool);
   }
 
