@@ -176,6 +176,7 @@ gst_decklink_src_init (GstDecklinkSrc * decklinksrc)
       GST_DEBUG_FUNCPTR (gst_decklink_src_video_src_query));
   gst_element_add_pad (GST_ELEMENT (decklinksrc), decklinksrc->videosrcpad);
 
+  GST_OBJECT_FLAG_SET (decklinksrc, GST_ELEMENT_FLAG_SOURCE);
 
   g_cond_init (&decklinksrc->cond);
   g_mutex_init (&decklinksrc->mutex);
@@ -361,6 +362,7 @@ gst_decklink_src_send_event (GstElement * element, GstEvent * event)
     case GST_EVENT_EOS:
       g_atomic_int_set (&src->pending_eos, TRUE);
       GST_INFO_OBJECT (src, "EOS pending");
+      g_cond_signal (&src->cond);
       result = TRUE;
       break;
       break;
@@ -709,12 +711,6 @@ gst_decklink_src_send_initial_events (GstDecklinkSrc * src)
   gst_event_set_group_id (event, group_id);
   gst_pad_push_event (src->videosrcpad, event);
 
-  /* segment */
-  gst_segment_init (&segment, GST_FORMAT_TIME);
-  event = gst_event_new_segment (&segment);
-  gst_pad_push_event (src->videosrcpad, gst_event_ref (event));
-  gst_pad_push_event (src->audiosrcpad, event);
-
   /* caps */
   gst_pad_push_event (src->audiosrcpad,
       gst_event_new_caps (gst_caps_new_simple ("audio/x-raw",
@@ -724,6 +720,12 @@ gst_decklink_src_send_initial_events (GstDecklinkSrc * src)
 
   gst_pad_push_event (src->videosrcpad,
       gst_event_new_caps (gst_decklink_mode_get_caps (src->mode)));
+
+  /* segment */
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  event = gst_event_new_segment (&segment);
+  gst_pad_push_event (src->videosrcpad, gst_event_ref (event));
+  gst_pad_push_event (src->audiosrcpad, event);
 }
 
 static void
@@ -744,7 +746,8 @@ gst_decklink_src_task (void *priv)
   GST_DEBUG_OBJECT (decklinksrc, "task");
 
   g_mutex_lock (&decklinksrc->mutex);
-  while (decklinksrc->video_frame == NULL && !decklinksrc->stop) {
+  while (decklinksrc->video_frame == NULL && !decklinksrc->stop &&
+      !decklinksrc->pending_eos) {
     g_cond_wait (&decklinksrc->cond, &decklinksrc->mutex);
   }
   video_frame = decklinksrc->video_frame;
@@ -762,6 +765,13 @@ gst_decklink_src_task (void *priv)
     return;
   }
 
+  if (g_atomic_int_compare_and_exchange (&decklinksrc->pending_eos, TRUE,
+      FALSE)) {
+    GST_INFO_OBJECT (decklinksrc, "EOS pending");
+    flow = GST_FLOW_EOS;
+    goto pause;
+  }
+
   /* warning on dropped frames */
   /* FIXME: post QoS message */
   if (decklinksrc->dropped_frames - decklinksrc->dropped_frames_old > 0) {
@@ -886,12 +896,6 @@ gst_decklink_src_task (void *priv)
   else
     flow = video_flow;
 
-  if (g_atomic_int_compare_and_exchange (&decklinksrc->pending_eos, TRUE,
-      FALSE)) {
-    GST_INFO_OBJECT (decklinksrc, "EOS pending");
-    flow = GST_FLOW_EOS;
-  }
-
   if (flow != GST_FLOW_OK)
     goto pause;
 
