@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef _GST_DECKLINK_SRC_H_
@@ -44,6 +44,11 @@ struct _GstDecklinkSrc
   GstPad *audiosrcpad;
   GstPad *videosrcpad;
 
+  gboolean  pending_eos;    /* ATOMIC */
+
+  gboolean  have_events;    /* ATOMIC */
+  GList    *pending_events; /* OBJECT_LOCK */
+
   IDeckLink *decklink;
   IDeckLinkInput *input;
   IDeckLinkConfiguration *config;
@@ -77,7 +82,7 @@ struct _GstDecklinkSrc
   GstDecklinkModeEnum mode;
   GstDecklinkConnectionEnum connection;
   GstDecklinkAudioConnectionEnum audio_connection;
-  int device;
+  int device_number;
 
 #ifdef _MSC_VER
   gboolean comInitialized;
