@@ -779,7 +779,8 @@ dvb_base_bin_handle_message (GstBin * bin, GstMessage * message)
 
   dvbbasebin = GST_DVB_BASE_BIN (bin);
 
-  if (GST_ELEMENT (message->src) == GST_ELEMENT (dvbbasebin->tsparse)) {
+  /* note: message->src might be a GstPad, so use element cast w/o typecheck */
+  if (GST_ELEMENT_CAST (message->src) == GST_ELEMENT (dvbbasebin->tsparse)) {
     GstMpegTsSection *section = gst_message_parse_mpegts_section (message);
 
     if (section) {
@@ -806,7 +807,7 @@ dvb_base_bin_handle_message (GstBin * bin, GstMessage * message)
 static void
 dvb_base_bin_pat_info_cb (DvbBaseBin * dvbbasebin, GstMpegTsSection * section)
 {
-  GArray *pat;
+  GPtrArray *pat;
   DvbBaseBinProgram *program;
   DvbBaseBinStream *stream;
   guint old_pmt_pid;
@@ -819,7 +820,7 @@ dvb_base_bin_pat_info_cb (DvbBaseBin * dvbbasebin, GstMpegTsSection * section)
   }
 
   for (i = 0; i < pat->len; i++) {
-    GstMpegTsPatProgram *patp = &g_array_index (pat, GstMpegTsPatProgram, i);
+    GstMpegTsPatProgram *patp = g_ptr_array_index (pat, i);
 
     program = dvb_base_bin_get_program (dvbbasebin, patp->program_number);
     if (program == NULL)
