@@ -17,18 +17,27 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <glib.h>
 #include <glib-object.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gst/gst.h>
 
+#include <gst/gst-i18n-plugin.h>
+
 #include "parsechannels.h"
 
+GST_DEBUG_CATEGORY_EXTERN (dvb_base_bin_debug);
+#define GST_CAT_DEFAULT dvb_base_bin_debug
+
 /* TODO:
  * Store the channels hash table around instead of constantly parsing it
  * Detect when the file changed on disk
@@ -36,7 +45,8 @@
 
 /* this will do zap style channels.conf only for the moment */
 static GHashTable *
-parse_channels_conf_from_file (GstElement * dvbbasebin, const gchar * filename)
+parse_channels_conf_from_file (GstElement * dvbbasebin, const gchar * filename,
+    GError ** error)
 {
   gchar *contents;
   gchar **lines;
@@ -56,6 +66,8 @@ parse_channels_conf_from_file (GstElement * dvbbasebin, const gchar * filename)
   GHashTable *res;
   GError *err = NULL;
 
+  GST_INFO_OBJECT (dvbbasebin, "parsing '%s'", filename);
+
   if (!g_file_get_contents (filename, &contents, NULL, &err))
     goto open_fail;
 
@@ -140,17 +152,21 @@ parse_channels_conf_from_file (GstElement * dvbbasebin, const gchar * filename)
 
 open_fail:
   {
-    GST_ELEMENT_ERROR (dvbbasebin, RESOURCE, READ, (NULL),
-        ("Opening channels configuration file failed : %s", filename,
-            err->message));
+    if (err->code == G_FILE_ERROR_NOENT) {
+      g_set_error (error, GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_NOT_FOUND,
+          _("Couldn't find DVB channel configuration file"));
+    } else {
+      g_set_error (error, GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_READ,
+          _("Couldn't load DVB channel configuration file: %s"), err->message);
+    }
     g_clear_error (&err);
     return NULL;
   }
 
 no_channels:
   {
-    GST_ELEMENT_ERROR (dvbbasebin, RESOURCE, READ, (NULL),
-        ("Channels configuration file doesn't contain any channels"));
+    g_set_error (error, GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+        _("DVB channel configuration file doesn't contain any channels"));
     g_hash_table_unref (res);
     return NULL;
   }
@@ -173,7 +189,8 @@ destroy_channels_hash (GHashTable * channels)
 }
 
 gboolean
-set_properties_for_channel (GstElement * dvbbasebin, const gchar * channel_name)
+set_properties_for_channel (GstElement * dvbbasebin,
+    const gchar * channel_name, GError ** error)
 {
   gboolean ret = FALSE;
   GHashTable *channels, *params;
@@ -183,13 +200,10 @@ set_properties_for_channel (GstElement * dvbbasebin, const gchar * channel_name)
 
   filename = g_strdup (g_getenv ("GST_DVB_CHANNELS_CONF"));
   if (filename == NULL) {
-    guint major, minor, micro, nano;
-
-    gst_version (&major, &minor, &micro, &nano);
-    filename = g_strdup_printf ("%s/gstreamer-%d.%d/dvb-channels.conf",
-        g_get_user_config_dir (), major, minor);
+    filename = g_build_filename (g_get_user_config_dir (),
+        "gstreamer-" GST_API_VERSION, "dvb-channels.conf", NULL);
   }
-  channels = parse_channels_conf_from_file (dvbbasebin, filename);
+  channels = parse_channels_conf_from_file (dvbbasebin, filename, error);
   g_free (filename);
 
   if (!channels)
@@ -418,9 +432,9 @@ beach:
 
 unknown_channel:
   {
-    GST_ELEMENT_ERROR (dvbbasebin, RESOURCE, READ, (NULL),
-        ("Couldn't find configuration properties for channel \"%s\"",
-            channel_name));
+    /* FIXME: is channel name guaranteed to be ASCII or UTF-8? */
+    g_set_error (error, GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_NOT_FOUND,
+        _("Couldn't find details for DVB channel %s"), channel_name);
     destroy_channels_hash (channels);
     return FALSE;
   }
