@@ -39,8 +39,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -957,7 +957,7 @@ qtwrapper_audio_decoder_base_init (QTWrapperAudioDecoderClass * klass)
   description =
       g_strdup_printf ("QTWrapper SCAudio wrapper for decoder: %s",
       GST_STR_NULL (info));
-  gst_element_class_set_details_simple (element_class,
+  gst_element_class_set_metadata (element_class,
       longname, "Codec/Decoder/Audio", description,
       "Fluendo <gstreamer@fluendo.com>, "
       "Pioneers of the Inevitable <songbird@songbirdnest.com>");
