@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <string.h>
@@ -289,8 +289,8 @@ assrender_suite (void)
   tcase_set_timeout (tc_chain, 120);
 
   suite_add_tcase (s, tc_chain);
-  tcase_skip_broken_test (tc_chain, test_assrender_basic_xRGB);
-  tcase_skip_broken_test (tc_chain, test_assrender_basic_I420);
+  tcase_add_test (tc_chain, test_assrender_basic_xRGB);
+  tcase_add_test (tc_chain, test_assrender_basic_I420);
 
   return s;
 }
