@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <unistd.h>
@@ -96,6 +96,16 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
+static GList *events = NULL;
+
+static gboolean
+collect_events_func (GstPad * pad, GstObject * parent, GstEvent * event)
+{
+  GST_LOG ("event: %" GST_PTR_FORMAT, event);
+  events = g_list_append (events, gst_event_ref (event));
+  return gst_pad_event_default (pad, parent, event);
+}
+
 static GstElement *
 setup_katedec (void)
 {
@@ -103,8 +113,9 @@ setup_katedec (void)
 
   GST_DEBUG ("setup_katedec");
   katedec = gst_check_setup_element ("katedec");
-  mydecsrcpad = gst_check_setup_src_pad (katedec, &srctemplate, NULL);
-  mydecsinkpad = gst_check_setup_sink_pad (katedec, &sinktemplate, NULL);
+  mydecsrcpad = gst_check_setup_src_pad (katedec, &srctemplate);
+  mydecsinkpad = gst_check_setup_sink_pad (katedec, &sinktemplate);
+  gst_pad_set_event_function (mydecsinkpad, collect_events_func);
   gst_pad_set_active (mydecsrcpad, TRUE);
   gst_pad_set_active (mydecsinkpad, TRUE);
 
@@ -114,9 +125,16 @@ setup_katedec (void)
 static void
 cleanup_katedec (GstElement * katedec)
 {
+  GList *l;
+
   GST_DEBUG ("cleanup_katedec");
   gst_element_set_state (katedec, GST_STATE_NULL);
 
+  for (l = events; l != NULL; l = l->next)
+    gst_event_unref (GST_EVENT (l->data));
+  g_list_free (events);
+  events = NULL;
+
   gst_pad_set_active (mydecsrcpad, FALSE);
   gst_pad_set_active (mydecsinkpad, FALSE);
   gst_check_teardown_src_pad (katedec);
@@ -131,8 +149,8 @@ setup_kateenc (void)
 
   GST_DEBUG ("setup_kateenc");
   kateenc = gst_check_setup_element ("kateenc");
-  myencsrcpad = gst_check_setup_src_pad (kateenc, &srctemplate, NULL);
-  myencsinkpad = gst_check_setup_sink_pad (kateenc, &sinktemplate, NULL);
+  myencsrcpad = gst_check_setup_src_pad (kateenc, &srctemplate);
+  myencsinkpad = gst_check_setup_sink_pad (kateenc, &sinktemplate);
   gst_pad_set_active (myencsrcpad, TRUE);
   gst_pad_set_active (myencsinkpad, TRUE);
 
@@ -159,8 +177,8 @@ setup_kateparse (void)
 
   GST_DEBUG ("setup_kateparse");
   kateparse = gst_check_setup_element ("kateparse");
-  myparsesrcpad = gst_check_setup_src_pad (kateparse, &srctemplate, NULL);
-  myparsesinkpad = gst_check_setup_sink_pad (kateparse, &sinktemplate, NULL);
+  myparsesrcpad = gst_check_setup_src_pad (kateparse, &srctemplate);
+  myparsesinkpad = gst_check_setup_sink_pad (kateparse, &sinktemplate);
   gst_pad_set_active (myparsesrcpad, TRUE);
   gst_pad_set_active (myparsesinkpad, TRUE);
 
@@ -187,8 +205,8 @@ setup_katetag (void)
 
   GST_DEBUG ("setup_katetag");
   katetag = gst_check_setup_element ("katetag");
-  mytagsrcpad = gst_check_setup_src_pad (katetag, &srctemplate, NULL);
-  mytagsinkpad = gst_check_setup_sink_pad (katetag, &sinktemplate, NULL);
+  mytagsrcpad = gst_check_setup_src_pad (katetag, &srctemplate);
+  mytagsinkpad = gst_check_setup_sink_pad (katetag, &sinktemplate);
   gst_pad_set_active (mytagsrcpad, TRUE);
   gst_pad_set_active (mytagsinkpad, TRUE);
 
@@ -212,6 +230,7 @@ static void
 check_buffers (guint expected, gboolean headers_in_caps)
 {
   GstBuffer *outbuffer;
+  GstMapInfo info;
   guint i, num_buffers;
   const int num_headers = 9;
   unsigned char packet_type;
@@ -222,23 +241,24 @@ check_buffers (guint expected, gboolean headers_in_caps)
   for (i = 0; i < num_buffers; ++i) {
     outbuffer = GST_BUFFER (buffers->data);
     fail_if (outbuffer == NULL);
-    fail_if (GST_BUFFER_SIZE (outbuffer) == 0);
+    fail_if (gst_buffer_get_size (outbuffer) == 0);
 
+    assert_equals_int (gst_buffer_map (outbuffer, &info, GST_MAP_READ), TRUE);
     if (i < num_headers) {
       /* different headers packets */
       packet_type = (0x80 | i);
-      fail_unless (GST_BUFFER_DATA (outbuffer)[0] == packet_type);
+      fail_unless (info.data[0] == packet_type);
       /* headers could be in caps, so would have an extra ref */
     } else if (i == num_buffers - 1) {
       /* eos data packet */
       packet_type = 0x7f;
-      fail_unless (GST_BUFFER_DATA (outbuffer)[0] == packet_type);
+      fail_unless (info.data[0] == packet_type);
     } else {
       /* data packet */
       packet_type = 0;
-      fail_unless (GST_BUFFER_DATA (outbuffer)[0] >= 0
-          && GST_BUFFER_DATA (outbuffer)[0] < 0x7f);
+      fail_unless (info.data[0] >= 0 && info.data[0] < 0x7f);
     }
+    gst_buffer_unmap (outbuffer, &info);
 
     buffers = g_list_remove (buffers, outbuffer);
 
@@ -255,9 +275,8 @@ GST_START_TEST (test_kate_typefind)
   GstBuffer *buf;
   GstCaps *caps = NULL;
 
-  buf = gst_buffer_new ();
-  GST_BUFFER_DATA (buf) = (guint8 *) kate_header_0x80;
-  GST_BUFFER_SIZE (buf) = sizeof (kate_header_0x80);
+  buf = gst_buffer_new_wrapped (g_memdup (kate_header_0x80,
+          sizeof (kate_header_0x80)), sizeof (kate_header_0x80));
   GST_BUFFER_OFFSET (buf) = 0;
 
   caps = gst_type_find_helper_for_buffer (NULL, buf, &prob);
@@ -279,6 +298,7 @@ GST_START_TEST (test_kate_empty_identification_header)
   GstElement *katedec;
   GstBuffer *inbuffer;
   GstBus *bus;
+  GstCaps *caps;
 
   katedec = setup_katedec ();
   bus = gst_bus_new ();
@@ -287,6 +307,10 @@ GST_START_TEST (test_kate_empty_identification_header)
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
 
+  caps = gst_caps_new_empty_simple ("subtitle/x-kate");
+  gst_check_setup_events (mydecsrcpad, katedec, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+
   inbuffer = gst_buffer_new_and_alloc (0);
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
 
@@ -312,10 +336,7 @@ GST_START_TEST (test_kate_identification_header)
   GstElement *katedec;
   GstBuffer *inbuffer;
   GstBus *bus;
-  GstMessage *message;
-  GstTagList *tag_list;
-  gchar *language;
-  gchar *title;
+  GstCaps *caps;
 
   katedec = setup_katedec ();
   fail_unless (gst_element_set_state (katedec,
@@ -323,9 +344,12 @@ GST_START_TEST (test_kate_identification_header)
       "could not set to playing");
   bus = gst_bus_new ();
 
-  inbuffer = gst_buffer_new_and_alloc (sizeof (kate_header_0x80));
-  memcpy (GST_BUFFER_DATA (inbuffer), kate_header_0x80,
-      sizeof (kate_header_0x80));
+  caps = gst_caps_new_empty_simple ("subtitle/x-kate");
+  gst_check_setup_events (mydecsrcpad, katedec, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
+
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x80,
+          sizeof (kate_header_0x80)), sizeof (kate_header_0x80));
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
   gst_buffer_ref (inbuffer);
 
@@ -337,9 +361,8 @@ GST_START_TEST (test_kate_identification_header)
   gst_buffer_unref (inbuffer);
   fail_unless (g_list_length (buffers) == 0);
 
-  inbuffer = gst_buffer_new_and_alloc (sizeof (kate_header_0x81));
-  memcpy (GST_BUFFER_DATA (inbuffer), kate_header_0x81,
-      sizeof (kate_header_0x81));
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x81,
+          sizeof (kate_header_0x81)), sizeof (kate_header_0x81));
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
   gst_buffer_ref (inbuffer);
 
@@ -349,21 +372,38 @@ GST_START_TEST (test_kate_identification_header)
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
   gst_buffer_unref (inbuffer);
   fail_unless (g_list_length (buffers) == 0);
-  /* there's a tag message waiting */
-  fail_if ((message = gst_bus_pop (bus)) == NULL);
-  gst_message_parse_tag (message, &tag_list);
-  fail_unless_equals_int (gst_tag_list_get_tag_size (tag_list,
-          GST_TAG_LANGUAGE_CODE), 1);
-  fail_unless (gst_tag_list_get_string (tag_list, GST_TAG_LANGUAGE_CODE,
-          &language));
-  fail_unless_equals_string (language, "en");
-  g_free (language);
-  fail_unless_equals_int (gst_tag_list_get_tag_size (tag_list, "title"), 1);
-  fail_unless (gst_tag_list_get_string (tag_list, GST_TAG_TITLE, &title));
-  fail_unless_equals_string (title, "Tiger");
-  g_free (title);
-  gst_tag_list_unref (tag_list);
-  gst_message_unref (message);
+
+  /* there should've been a tag event */
+  {
+    gboolean found_tags = FALSE;
+    GList *l;
+
+    for (l = events; l != NULL; l = l->next) {
+      GstEvent *event = GST_EVENT (l->data);
+
+      if (GST_EVENT_TYPE (event) == GST_EVENT_TAG) {
+        GstTagList *tags = NULL;
+        gchar *language;
+        gchar *title;
+
+        found_tags = TRUE;
+        gst_event_parse_tag (event, &tags);
+        fail_unless (tags != NULL);
+        fail_unless (gst_tag_list_get_scope (tags) == GST_TAG_SCOPE_STREAM);
+        fail_unless_equals_int (gst_tag_list_get_tag_size (tags,
+                GST_TAG_LANGUAGE_CODE), 1);
+        fail_unless (gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE,
+                &language));
+        fail_unless_equals_string (language, "en");
+        g_free (language);
+        fail_unless_equals_int (gst_tag_list_get_tag_size (tags, "title"), 1);
+        fail_unless (gst_tag_list_get_string (tags, GST_TAG_TITLE, &title));
+        fail_unless_equals_string (title, "Tiger");
+        g_free (title);
+      }
+    }
+    fail_unless (found_tags);
+  }
 
   /* cleanup */
   gst_bus_set_flushing (bus, TRUE);
@@ -414,9 +454,9 @@ GST_START_TEST (test_kate_encode_empty)
   GST_BUFFER_DURATION (inbuffer) = 5 * GST_SECOND;
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
 
-  caps = gst_caps_from_string ("text/plain");
+  caps = gst_caps_from_string ("text/x-raw, format=utf8");
   fail_unless (caps != NULL);
-  gst_buffer_set_caps (inbuffer, caps);
+  gst_check_setup_events (myencsrcpad, kateenc, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
 
   gst_element_set_bus (kateenc, bus);
@@ -454,22 +494,23 @@ GST_START_TEST (test_kate_encode_simple)
       "could not set to playing");
   bus = gst_bus_new ();
 
-  inbuffer = gst_buffer_new_and_alloc (strlen (test_string) + 1);
-  memcpy (GST_BUFFER_DATA (inbuffer), test_string, strlen (test_string) + 1);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (test_string,
+          strlen (test_string) + 1), strlen (test_string) + 1);
+
   GST_BUFFER_TIMESTAMP (inbuffer) = GST_BUFFER_OFFSET (inbuffer) =
       1 * GST_SECOND;
   GST_BUFFER_DURATION (inbuffer) = 5 * GST_SECOND;
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
 
-  caps = gst_caps_from_string ("text/plain");
+  caps = gst_caps_from_string ("text/x-raw, format=utf8");
   fail_unless (caps != NULL);
-  gst_buffer_set_caps (inbuffer, caps);
+  gst_check_setup_events (myencsrcpad, kateenc, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   gst_buffer_ref (inbuffer);
 
   gst_element_set_bus (kateenc, bus);
   /* pushing gives away my reference ... */
-  fail_unless (gst_pad_push (myencsrcpad, inbuffer) == GST_FLOW_OK);
+  fail_unless_equals_int (gst_pad_push (myencsrcpad, inbuffer), GST_FLOW_OK);
   /* ... and nothing ends up on the global buffer list */
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
   gst_buffer_unref (inbuffer);
@@ -487,7 +528,7 @@ GST_START_TEST (test_kate_encode_simple)
   gst_element_set_bus (kateenc, NULL);
   gst_object_unref (GST_OBJECT (bus));
   cleanup_kateenc (kateenc);
-  g_list_free (buffers);
+  gst_check_drop_buffers ();
 }
 
 GST_END_TEST;
@@ -507,8 +548,9 @@ GST_START_TEST (test_kate_encode_spu)
       "could not set to playing");
   bus = gst_bus_new ();
 
-  inbuffer = gst_buffer_new_and_alloc (sizeof (kate_spu));
-  memcpy (GST_BUFFER_DATA (inbuffer), kate_spu, sizeof (kate_spu));
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_spu, sizeof (kate_spu)),
+      sizeof (kate_spu));
+
   GST_BUFFER_TIMESTAMP (inbuffer) = GST_BUFFER_OFFSET (inbuffer) =
       1 * GST_SECOND;
   GST_BUFFER_DURATION (inbuffer) = 5 * GST_SECOND;
@@ -516,7 +558,7 @@ GST_START_TEST (test_kate_encode_spu)
 
   caps = gst_caps_from_string ("subpicture/x-dvd");
   fail_unless (caps != NULL);
-  gst_buffer_set_caps (inbuffer, caps);
+  gst_check_setup_events (myencsrcpad, kateenc, caps, GST_FORMAT_TIME);
   gst_caps_unref (caps);
   gst_buffer_ref (inbuffer);
 
@@ -540,7 +582,7 @@ GST_START_TEST (test_kate_encode_spu)
   gst_element_set_bus (kateenc, NULL);
   gst_object_unref (GST_OBJECT (bus));
   cleanup_kateenc (kateenc);
-  g_list_free (buffers);
+  gst_check_drop_buffers ();
 }
 
 GST_END_TEST;
@@ -550,6 +592,7 @@ GST_START_TEST (test_kate_encode_keepalives)
   GstElement *kateenc;
   GstBus *bus;
   guint i, round;
+  GstSegment segment;
   enum
   { n_keepalives = 1000 };
   static const struct
@@ -576,13 +619,17 @@ GST_START_TEST (test_kate_encode_keepalives)
     g_object_set (kateenc, "keepalive-min-time", cfg[round].keepalive_min_time,
         NULL);
 
+    gst_pad_push_event (myencsrcpad, gst_event_new_stream_start ("test"));
+
     /* the second one here should not emit a keepalive since the time since last packet
        is less than the keepalive delay */
     for (i = 1; i <= n_keepalives; ++i) {
       gint64 t = i * GST_SECOND;
+      gst_segment_init (&segment, GST_FORMAT_TIME);
+      segment.start = t;
+      segment.position = 0;
       fail_unless (gst_pad_push_event (myencsrcpad,
-              gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_TIME, t, -1,
-                  0)) == TRUE);
+              gst_event_new_segment (&segment)) == TRUE);
     }
 
     fail_unless (gst_pad_push_event (myencsrcpad,
@@ -600,54 +647,49 @@ GST_START_TEST (test_kate_encode_keepalives)
     gst_element_set_bus (kateenc, NULL);
     gst_object_unref (GST_OBJECT (bus));
     cleanup_kateenc (kateenc);
-    g_list_free (buffers);
+    gst_check_drop_buffers ();
   }
 }
 
 GST_END_TEST;
 
 static void
-test_kate_send_headers (GstPad * pad)
+test_kate_send_headers (GstElement * element, GstPad * pad)
 {
   GstBuffer *inbuffer;
   GstCaps *caps;
+  GstMapInfo info;
   int i;
 
-  caps = gst_caps_new_simple ("subtitle/x-kate", NULL);
+  caps = gst_caps_new_simple ("subtitle/x-kate", NULL, NULL);
+  gst_check_setup_events (pad, element, caps, GST_FORMAT_TIME);
+  gst_caps_unref (caps);
 
   /* push headers */
-  inbuffer = gst_buffer_new ();
-  gst_buffer_set_caps (inbuffer, caps);
-  GST_BUFFER_DATA (inbuffer) = (guint8 *) kate_header_0x80;
-  GST_BUFFER_SIZE (inbuffer) = sizeof (kate_header_0x80);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x80,
+          sizeof (kate_header_0x80)), sizeof (kate_header_0x80));
   GST_BUFFER_OFFSET (inbuffer) = GST_BUFFER_OFFSET_END (inbuffer) = 0;
   fail_unless_equals_int (gst_pad_push (pad, inbuffer), GST_FLOW_OK);
 
-  inbuffer = gst_buffer_new ();
-  gst_buffer_set_caps (inbuffer, caps);
-  GST_BUFFER_DATA (inbuffer) = (guint8 *) kate_header_0x81;
-  GST_BUFFER_SIZE (inbuffer) = sizeof (kate_header_0x81);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x81,
+          sizeof (kate_header_0x81)), sizeof (kate_header_0x81));
   GST_BUFFER_OFFSET (inbuffer) = GST_BUFFER_OFFSET_END (inbuffer) = 0;
   fail_unless_equals_int (gst_pad_push (pad, inbuffer), GST_FLOW_OK);
 
   for (i = 2; i < 8; ++i) {
-    inbuffer = gst_buffer_new_and_alloc (sizeof (kate_header_0x8x));
-    gst_buffer_set_caps (inbuffer, caps);
-    memcpy (GST_BUFFER_DATA (inbuffer), (guint8 *) kate_header_0x8x,
-        sizeof (kate_header_0x8x));
-    GST_BUFFER_DATA (inbuffer)[0] = 0x80 | i;
+    inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x8x,
+            sizeof (kate_header_0x8x)), sizeof (kate_header_0x8x));
+    fail_if (gst_buffer_map (inbuffer, &info, GST_MAP_WRITE) != TRUE);
+    info.data[0] = 0x80 | i;
+    gst_buffer_unmap (inbuffer, &info);
     GST_BUFFER_OFFSET (inbuffer) = GST_BUFFER_OFFSET_END (inbuffer) = 0;
     fail_unless_equals_int (gst_pad_push (pad, inbuffer), GST_FLOW_OK);
   }
 
-  inbuffer = gst_buffer_new ();
-  gst_buffer_set_caps (inbuffer, caps);
-  GST_BUFFER_DATA (inbuffer) = (guint8 *) kate_header_0x88;
-  GST_BUFFER_SIZE (inbuffer) = sizeof (kate_header_0x88);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x88,
+          sizeof (kate_header_0x88)), sizeof (kate_header_0x88));
   GST_BUFFER_OFFSET (inbuffer) = GST_BUFFER_OFFSET_END (inbuffer) = 0;
   fail_unless_equals_int (gst_pad_push (pad, inbuffer), GST_FLOW_OK);
-
-  gst_caps_unref (caps);
 }
 
 GST_START_TEST (test_kate_parse)
@@ -664,12 +706,11 @@ GST_START_TEST (test_kate_parse)
 
   gst_element_set_bus (kateparse, bus);
 
-  test_kate_send_headers (myparsesrcpad);
+  test_kate_send_headers (kateparse, myparsesrcpad);
 
   /* push a text packet */
-  inbuffer = gst_buffer_new ();
-  GST_BUFFER_DATA (inbuffer) = (guint8 *) kate_header_0x00;
-  GST_BUFFER_SIZE (inbuffer) = sizeof (kate_header_0x00);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x00,
+          sizeof (kate_header_0x00)), sizeof (kate_header_0x00));
   GST_BUFFER_TIMESTAMP (inbuffer) = GST_BUFFER_OFFSET (inbuffer) =
       1 * GST_SECOND;
   GST_BUFFER_DURATION (inbuffer) = 5 * GST_SECOND;
@@ -677,9 +718,8 @@ GST_START_TEST (test_kate_parse)
   fail_unless_equals_int (gst_pad_push (myparsesrcpad, inbuffer), GST_FLOW_OK);
 
   /* push a eos packet */
-  inbuffer = gst_buffer_new ();
-  GST_BUFFER_DATA (inbuffer) = (guint8 *) kate_header_0x7f;
-  GST_BUFFER_SIZE (inbuffer) = sizeof (kate_header_0x7f);
+  inbuffer = gst_buffer_new_wrapped (g_memdup (kate_header_0x7f,
+          sizeof (kate_header_0x7f)), sizeof (kate_header_0x7f));
   GST_BUFFER_TIMESTAMP (inbuffer) = GST_BUFFER_OFFSET (inbuffer) =
       6 * GST_SECOND;
   GST_BUFFER_DURATION (inbuffer) = 0;
@@ -702,7 +742,8 @@ GST_START_TEST (test_kate_parse)
   gst_element_set_bus (kateparse, NULL);
   gst_object_unref (GST_OBJECT (bus));
   cleanup_kateparse (kateparse);
-  g_list_free (buffers);
+  g_list_foreach (buffers, (GFunc) gst_buffer_unref, NULL);
+  gst_check_drop_buffers ();
 }
 
 GST_END_TEST;
@@ -713,6 +754,7 @@ GST_START_TEST (test_kate_tag_passthrough)
   GstBus *bus;
   GstBuffer *outbuffer;
   GList *list;
+  GstMapInfo info;
 
   katetag = setup_katetag ();
   fail_unless (gst_element_set_state (katetag,
@@ -722,7 +764,7 @@ GST_START_TEST (test_kate_tag_passthrough)
 
   gst_element_set_bus (katetag, bus);
 
-  test_kate_send_headers (mytagsrcpad);
+  test_kate_send_headers (katetag, mytagsrcpad);
 
   /* signal eos */
   fail_unless (gst_pad_push_event (mytagsrcpad, gst_event_new_eos ()) == TRUE);
@@ -740,19 +782,21 @@ GST_START_TEST (test_kate_tag_passthrough)
   list = g_list_nth (buffers, 0);
   fail_unless (list != NULL);
   outbuffer = list->data;
-  fail_unless_equals_int (GST_BUFFER_SIZE (outbuffer),
-      sizeof (kate_header_0x80));
-  fail_unless_equals_int (memcmp (GST_BUFFER_DATA (outbuffer), kate_header_0x80,
+  fail_if (gst_buffer_map (outbuffer, &info, GST_MAP_READ) != TRUE);
+  fail_unless_equals_int (info.size, sizeof (kate_header_0x80));
+  fail_unless_equals_int (memcmp (info.data, kate_header_0x80,
           sizeof (kate_header_0x80)), 0);
+  gst_buffer_unmap (outbuffer, &info);
 
   /* check comment header is unchanged */
   list = g_list_nth (buffers, 1);
   fail_unless (list != NULL);
   outbuffer = list->data;
-  fail_unless_equals_int (GST_BUFFER_SIZE (outbuffer),
-      sizeof (kate_header_0x81));
-  fail_unless_equals_int (memcmp (GST_BUFFER_DATA (outbuffer), kate_header_0x81,
+  fail_if (gst_buffer_map (outbuffer, &info, GST_MAP_READ) != TRUE);
+  fail_unless_equals_int (info.size, sizeof (kate_header_0x81));
+  fail_unless_equals_int (memcmp (info.data, kate_header_0x81,
           sizeof (kate_header_0x81)), 0);
+  gst_buffer_unmap (outbuffer, &info);
 
   /* all headers should have been emitted, but no particular packets */
   check_buffers (0, TRUE);
@@ -762,7 +806,7 @@ GST_START_TEST (test_kate_tag_passthrough)
   gst_element_set_bus (katetag, NULL);
   gst_object_unref (GST_OBJECT (bus));
   cleanup_katetag (katetag);
-  g_list_free (buffers);
+  gst_check_drop_buffers ();
 }
 
 GST_END_TEST;
@@ -772,6 +816,7 @@ GST_START_TEST (test_kate_tag)
   GstElement *katetag;
   GstBus *bus;
   GstBuffer *outbuffer;
+  GstMapInfo info;
 
   katetag = setup_katetag ();
   fail_unless (gst_element_set_state (katetag,
@@ -784,7 +829,7 @@ GST_START_TEST (test_kate_tag)
   g_object_set (katetag, "language", "cy", NULL);
   g_object_set (katetag, "category", "subtitles", NULL);
 
-  test_kate_send_headers (mytagsrcpad);
+  test_kate_send_headers (katetag, mytagsrcpad);
 
   /* signal eos */
   fail_unless (gst_pad_push_event (mytagsrcpad, gst_event_new_eos ()) == TRUE);
@@ -797,10 +842,11 @@ GST_START_TEST (test_kate_tag)
   fail_unless (g_list_length (buffers) >= 1);
   outbuffer = GST_BUFFER (buffers->data);
   fail_if (outbuffer == NULL);
-  fail_if (GST_BUFFER_SIZE (outbuffer) != 64);
-  fail_if (strcmp ((const char *) GST_BUFFER_DATA (outbuffer) + 32, "cy"));
-  fail_if (strcmp ((const char *) GST_BUFFER_DATA (outbuffer) + 48,
-          "subtitles"));
+  assert_equals_int (gst_buffer_map (outbuffer, &info, GST_MAP_READ), TRUE);
+  fail_if (info.size != 64);
+  fail_if (strcmp ((const char *) info.data + 32, "cy"));
+  fail_if (strcmp ((const char *) info.data + 48, "subtitles"));
+  gst_buffer_unmap (outbuffer, &info);
 
   /* all headers should have been emitted, but no particular packets */
   check_buffers (0, TRUE);
@@ -810,7 +856,7 @@ GST_START_TEST (test_kate_tag)
   gst_element_set_bus (katetag, NULL);
   gst_object_unref (GST_OBJECT (bus));
   cleanup_katetag (katetag);
-  g_list_free (buffers);
+  gst_check_drop_buffers ();
 }
 
 GST_END_TEST;
@@ -823,7 +869,6 @@ kate_suite (void)
 
   suite_add_tcase (s, tc_chain);
 
-#define X if (0)
   tcase_add_test (tc_chain, test_kate_typefind);
   tcase_add_test (tc_chain, test_kate_empty_identification_header);
   tcase_add_test (tc_chain, test_kate_identification_header);
@@ -835,7 +880,6 @@ kate_suite (void)
   tcase_add_test (tc_chain, test_kate_parse);
   tcase_add_test (tc_chain, test_kate_tag_passthrough);
   tcase_add_test (tc_chain, test_kate_tag);
-#undef X
 
   return s;
 }
